(function (window){

	var HAPUtils = function(){};

	HAPUtils.isEmpty = function(str){
		return str.replace(/^\s+|\s+$/g, '').length == 0;
	}

	HAPUtils.isNumber = function(n){
	    return !isNaN(parseFloat(n)) && isFinite(n);
	}

	HAPUtils.isMobile = function(){
		return (/Android|webOS|iPhone|iPad|iPod|sony|BlackBerry/i.test(navigator.userAgent));
	}

	HAPUtils.isChrome = function(){
		return !!window.chrome && (!!window.chrome.webstore || !!window.chrome.runtime)
	}

	HAPUtils.isSafari = function() {
		return Object.prototype.toString.call(window.HTMLElement).indexOf('Constructor') > 0;
	}

	HAPUtils.isIE = function(){
		var ie_check = getInternetExplorerVersion();
		if (ie_check != -1)return true;
		else return false;
	}

	function getInternetExplorerVersion(){
	    var rv = -1;
	    if(navigator.appName == 'Microsoft Internet Explorer'){
			var ua = navigator.userAgent;
			var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
			if(re.exec(ua) != null)rv = parseFloat( RegExp.$1 );
	    }else if(navigator.appName == 'Netscape'){
			var ua = navigator.userAgent;
			var re  = new RegExp("Trident/.*rv:([0-9]{1,}[\.0-9]{0,})");
			if(re.exec(ua) != null)rv = parseFloat( RegExp.$1 );
	    }
	    return rv;
	}

	HAPUtils.isIOS = function(){
		return navigator.userAgent.match(/(iPad|iPhone|iPod)/g);
	}

	HAPUtils.isAndroid = function() {
		return navigator.userAgent.indexOf("Android") > -1;
	}

	HAPUtils.hasDownloadSupport = function(){  
		return ("download" in document.createElement("a"));
    }

    HAPUtils.qualifyURL = function(url) {
		var a = document.createElement('a');
		a.href = url;
		return a.href;
	}

	HAPUtils.relativePath = function(s){
    	//https://stackoverflow.com/questions/10687099/how-to-test-if-a-url-string-is-absolute-or-relative
		var r = new RegExp('^(?:[a-z]+:)?//', 'i');
		return r.test(s);
	}

	HAPUtils.closestNumber = function(arr, goal){
		var closest = arr.reduce(function(prev, curr) {
	  		return (Math.abs(curr - goal) < Math.abs(prev - goal) ? curr : prev);
		});
		return closest;
	}

    HAPUtils.hasLocalStorage = function() {
	    try{
			return 'localStorage' in window && window['localStorage'] !== null;
	    }catch(e){
			return false;
	    }
	}

	HAPUtils.volumeCanBeSet = function() {
		var audio = document.createElement("audio");
		if(!audio) return false;
		audio.volume = 0;
		return audio.volume == 0 ? true : false;
	};

	HAPUtils.randomiseArray = function(num) {
		var arr = [], randomArr = [], i, j, randomIndex;
		for(i = 0; i < num; i++){
			arr[i] = i;
		}
		for(j = 0; j < num; j++){
			randomIndex = Math.round(Math.random()*(arr.length-1));
			randomArr[j] = arr[randomIndex];
			arr.splice(randomIndex, 1);
		}
		return randomArr;
	}

	HAPUtils.valueLimit = function(val, min, max) {
	    return val < min ? min : (val > max ? max : val);
	}

	HAPUtils.sortArray = function(arr, sortArr) {
		var i, len = arr.length, result = [];
		for(i=0; i<len; i++) {
		   result[i] = arr[sortArr[i]];
		}
		return result;
	}

	HAPUtils.shuffleArray = function(a) {
	    for (let i = a.length - 1; i > 0; i--) {
	        const j = Math.floor(Math.random() * (i + 1));
	        [a[i], a[j]] = [a[j], a[i]];
	    }
	    return a;
	}

	HAPUtils.keysrt = function(arr, key, reverse) {
		var order = 1;
		if(reverse)order = -1;
		return arr.sort(function(a, b) {
			var x = a[key]; var y = b[key];
			return order * ((x < y) ? -1 : ((x > y) ? 1 : 0));
		});
	}

	HAPUtils.keysrt2 = function(arr, prop, key, reverse) {
	    var sortOrder = 1;
	    if(reverse)sortOrder = -1;
	    return arr.sort(function(a, b) {
	        var x = a[prop][key]; var y = b[prop][key];
	        return sortOrder * ((x < y) ? -1 : ((x > y) ? 1 : 0));
	    });
	}

	HAPUtils.parseXML = function(xml) {
		if(window.ActiveXObject && window.GetObject) {
			var dom = new ActiveXObject('Microsoft.XMLDOM');
			dom.loadXML(xml);
			return dom;
		}
		if(window.DOMParser){
			return new DOMParser().parseFromString(xml, 'text/xml');
		}else{
			throw new Error('No XML parser available');
		}
	}

	HAPUtils.formatTime = function(seconds) {
		var sec_num = parseInt(seconds, 10); 
	    var hours   = Math.floor(sec_num / 3600);
	    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
	    var seconds = sec_num - (hours * 3600) - (minutes * 60);
	    if(hours > 0){
	    	if (hours   < 10) {hours   = "0"+hours;}
		    if (minutes < 10) {minutes = "0"+minutes;}
		    if (seconds < 10) {seconds = "0"+seconds;}
		    return hours+':'+minutes+':'+seconds;
	    }else{
		    if (minutes < 10) {minutes = "0"+minutes;}
		    if (seconds < 10) {seconds = "0"+seconds;}
		    return minutes+':'+seconds;
	    }
	}	

	HAPUtils.formatNumber = function(i) {
		var s;
		if(i < 9)s = "0" + (i + 1);
		else s = i + 1;
		return s;
	} 

	HAPUtils.nFormatter = function(num, digits) {
	    var si = [
		    { value: 1E18, symbol: "E" },
		    { value: 1E15, symbol: "P" },
		    { value: 1E12, symbol: "T" },
		    { value: 1E9,  symbol: "G" },
		    { value: 1E6,  symbol: "M" },
		    { value: 1E3,  symbol: "k" }
	    ], rx = /\.0+$|(\.[0-9]*[1-9])0+$/, i;

	    for (i = 0; i < si.length; i++) {
		    if (num >= si[i].value) {
		        return (num / si[i].value).toFixed(digits).replace(rx, "$1") + si[i].symbol;
		    }
	    }
 	    return num.toFixed(digits).replace(rx, "$1");
	}

	HAPUtils.hmsToSecondsOnly = function(str) {//This function handels "HH:MM:SS" as well as "MM:SS" or "SS".
		var p = str.split(':'),
			s = 0, m = 1;
		while (p.length > 0) {
			s += m * parseInt(p.pop());
			m *= 60;
		}
		return s;
	}

	HAPUtils.canPlayMp3 = function() {
		var a = document.createElement('audio');
		return !!(a.canPlayType && a.canPlayType('audio/mpeg;').replace(/no/, ''));
	}

	HAPUtils.canPlayWav = function() {
		var a = document.createElement('audio');
		return !!(a.canPlayType && a.canPlayType('audio/wav;').replace(/no/, ''));
	}

	HAPUtils.canPlayAac = function() {
		var a = document.createElement('audio');
		return !!(a.canPlayType && a.canPlayType('audio/aac;').replace(/no/, ''));
	}

	HAPUtils.canPlayOgg = function() {
		var a = document.createElement('audio');
		return !!(a.canPlayType && a.canPlayType('audio/ogg; codecs="vorbis"').replace(/no/, ''));
	}

	HAPUtils.hasCanvas = function(){
		return Boolean(document.createElement("canvas"));
	};

	HAPUtils.getElementOffsetTop = function(el) {
	    var boundingClientRect = el.getBoundingClientRect();
	    var bodyEl = document.body;
	    var docEl = document.documentElement;
	    var scrollTop = window.pageYOffset || docEl.scrollTop || bodyEl.scrollTop;
	    var clientTop = docEl.clientTop || bodyEl.clientTop || 0;
	    return Math.round(boundingClientRect.bottom - 100 + scrollTop - clientTop);
	}

	HAPUtils.getScrollTop = function(el) {
	    var docEl = document.documentElement;
  		return (window.pageYOffset || docEl.scrollTop) - (docEl.clientTop || 0);
	}

	HAPUtils.getEvents = function() {

		var events = {};

	   	if("ontouchstart" in window){
			events.downEvent = "touchstart.ap mousedown.ap";
			events.moveEvent = "touchmove.ap mousemove.ap";
			events.upEvent = "touchend.ap mouseup.ap";
		}else{
			if(window.PointerEvent){
				events.downEvent = "pointerdown.ap";
				events.moveEvent = "pointermove.ap";
				events.upEvent = "pointerup.ap";
			}else{
				events.downEvent = "mousedown.ap";
				events.moveEvent = "mousemove.ap";
				events.upEvent = "mouseup.ap";
			}
		}

  		return events;
	}

	HAPUtils.getUrlParameter = function(k) {
		var p={};
		window.location.search.replace(/[?&]+([^=&]+)=([^&]*)/gi,function(s,k,v){p[k]=v})
		return k?p[k]:p;
	};

	window.HAPUtils = HAPUtils;

}(window));	


(function (window, $){
	"use strict"
	var HAPPlaylistManager = function (data){
		
		var self = this,
		loopState = data.loop,
		randomPlay = data.random,
		playlistItems,
		lastInOrder = false,
		counter = -1,
		lastPlayedFromPlaylistClick,//last played on click.
		lastRandomCounter,//last played random media in random playlist.
		randomPaused = false,//when random is playing and we interrupt it by click on the playlist.
		randomArr = [],
		playlistSelect = false;//prevent geting counter from randomArr on playlist click (get 'normal' counter instead)		
		
		//set counter to specific number or add it to the currect counter value		 
		this.setCounter = function(value, _add) {
			if (typeof _add === 'undefined') _add = true;
			if(_add){
				counter += parseInt(value, 10);
			}else{
				counter = parseInt(value, 10);
			}
			checkCounter();
		}
		this.getCounter = function() {
			var i;
			if(randomPlay){
				if(!playlistSelect){
					i = randomArr[counter];
				}else{
					i = counter;
				}
			}else{
				i = counter;
			}
			return i;
		}
		this.advanceHandler = function(a) {
			playlistSelect = false;//reset
			if(randomPaused){
				handleRandomPaused(a);
			}else{
				self.setCounter(a);
			}
		}
		this.processPlaylistRequest = function(id) {
			playlistSelect = false;//reset
			if(randomPlay){
				playlistSelect = true;
				lastPlayedFromPlaylistClick = id;//always remember last played on each click.
				if(!randomPaused){
					lastRandomCounter = counter;
					randomPaused = true;//needs to stay until random play comes back again! So that the above reference to last random counter doesnt get lost. (if we constantly clicking playlist)
				}
			}
			self.setCounter(id, false);
		}
		this.setPlaylistItems = function(val, resetCounter) {
			if(typeof resetCounter === 'undefined') resetCounter = true;
			if(resetCounter)counter = -1;
			playlistItems = val;
			if(randomPlay) makeRandomList();
		}
		this.reSetCounter = function(num) {
			if(typeof num === 'undefined'){
				 counter = -1;
			}else{//set counter to specific number
				var n = parseInt(num,10);
				if(playlistItems){
					if(n > playlistItems - 1){
						n = playlistItems - 1;
					}else if(n < 0){
						n = 0;
					}
					counter = n;
				}else{
					counter = -1;
				}
			}
		}
		this.setRandom = function(val) {
			randomPlay = val;
			if(randomPlay) makeRandomList();
			randomChange();
		}
		this.setLooping = function(val) {
			loopState = val;
		}
		this.getPosition = function(val) {
			return randomArr.indexOf(val);
		}
		
		//exiting randomPaused and going back to random mode
		function handleRandomPaused(a) {
			//just an exit out of randomPaused (because of a playlist click) and back to random again
			randomPaused = false;//reset before because of the getCounter()
			
			if(lastRandomCounter + a > playlistItems - 1){
				counter = playlistItems - 1;
				$(self).trigger('HAPPlaylistManager.COUNTER_READY', self.getCounter());
				return;
			} else if( lastRandomCounter + a < 0){
				counter = 0;
				$(self).trigger('HAPPlaylistManager.COUNTER_READY', self.getCounter());
				return;
			}
			self.setCounter(lastRandomCounter + a, false);
		}
		function randomChange() {//when random is turned on / off
			if(randomPlay){
				activeIndexFirst();
				counter = 0;//we have to do it like this, because with (setCounter(0, false)) media starts to play from the beginning if its already playing. (when random requested)
				//we need to say this on the every beginning of random to redirect the counter from wherever the currently is to 0, so that it becomes first index in randomArr. (after we have moved active index to beginning of randomArr)
				
			}else{
				//we are not going through setCounter here because its just getting out of random mode, and its not changing counter, it just stays where it is (playing or not)
				if(randomPaused){
					counter = lastPlayedFromPlaylistClick;
					randomPaused = false;//when random mode stops randomPaused stops also.
				}else{
					counter = randomArr[counter];//when we turn off random we need to set counter to the value of the current counter in randomArr, so if the counter is 1, and thats value 3 in randomArr for example, we want the active counter to stay 3, not 1, and next to go to 4, not 2.
				}
			}
		}
		function checkCounter() {
			if(isNaN(counter)){
				alert('HAPPlaylistManager message: No active media, counter = ' + counter);
				return;
			}
			//reset
			lastInOrder = false;
			
			if(loopState == 'playlist' || loopState == 'single'){
				if(randomPlay){
					
					if(counter > playlistItems - 1){//moving fowards
						counter = randomArr[ playlistItems - 1];//remember counter for comparison
						makeRandomList();
						_firstIndexCheck();
						counter = 0;
						
					}else if(counter < 0){//moving backwards
						counter = randomArr[0];//remember counter for comparison
						makeRandomList();
						lastIndexCheck();
						counter = playlistItems - 1;
					}
					
				}else{//random off
					if(counter > playlistItems - 1){
						counter = 0;
					}else if( counter < 0){
						counter = playlistItems - 1;
					}
				}
				
				$(self).trigger('HAPPlaylistManager.COUNTER_READY', self.getCounter());
				
			}else if(loopState == 'off'){//looping off
				
				if(counter > playlistItems - 1){
					counter = playlistItems - 1;
					lastInOrder = true;//last item
				}else if(counter < 0){
					counter = 0;
					lastInOrder = true;//last item
				}
				
				if(!lastInOrder){
					$(self).trigger('HAPPlaylistManager.COUNTER_READY', self.getCounter());
				}else{
					$(self).trigger('HAPPlaylistManager.PLAYLIST_END');
				}
			}
			
		}
		function makeRandomList() {
			randomArr = HAPUtils.randomiseArray(playlistItems);
			console.log(randomArr)
		}
		function _firstIndexCheck () {
			//check that first item in newly generated random array isnt equal to last active item.
			if(randomArr[0] == counter){//if yes, put it at the last place in array.
				var i = randomArr.splice(0,1);
				randomArr.push(i);
			}
		}
		function lastIndexCheck() {
			if(randomArr[playlistItems - 1] == counter){//if yes, put it at the first place in array.
				var i = randomArr.splice(playlistItems - 1,1);
				randomArr.unshift(i);
			}
		}
		function activeIndexFirst() {//when going into random (playing or not) put currently active index on the first place of random array.
			var i,len = randomArr.length, j;
			for(i = 0; i < len; i++){
				if(randomArr[i] == counter){
					if(i == 0){//if its already on the first place no need for action.
						break;
					}
					j = randomArr.splice(i,1);
					randomArr.unshift(parseInt(j,10));
					break;
				}
			}
		} 
	
	};
	
	window.HAPPlaylistManager = HAPPlaylistManager;	

}(window,jQuery));

(function (window, $){
	"use strict"	
	var HAPPlaybackRateSlider = function (data){

		var self = this,
		_doc = $(document),
		_window = $(window),
		wrapper = data.wrapper,
		isPressed,
		isVertical = data.isVertical,
		prop = isVertical ? 'height' : 'width',
		tooltip = data.tooltip,
		sliderBg = data.sliderBg,
		sliderLevel = data.sliderLevel,
		seekbar = data.seekbar,
		rangeSize = isVertical ? sliderBg.height() : sliderBg.width(),
		events = HAPUtils.getEvents(),
		min = data.settings.playbackRateMin, 
		max = data.settings.playbackRateMax

	

		seekbar.on(events.downEvent,function(e){
			onDragStart(e);
			return false;		
		});

		function onDragStart(e) {
			if(!isPressed){					
				var point;
				if(e.type == 'touchstart'){
					var currTouches = e.originalEvent.touches;
					if(currTouches && currTouches.length > 0) {
						point = currTouches[0];
					}else{	
						return false;						
					}
				}else{
					point = e;								
					e.preventDefault();						
				}
				isPressed = true;
				_doc.on(events.moveEvent, function(e) { onDragMove(e); }).on(events.upEvent, function(e) { onDragRelease(e); });		
			}
			return false;	
		}
					
		function onDragMove(e) {	
			var point;
			if(e.type == 'touchmove'){
				var touches;
				if(e.originalEvent.touches && e.originalEvent.touches.length) {
					touches = e.originalEvent.touches;
				}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
					touches = e.originalEvent.changedTouches;
				}else{
					return false;
				}
				// If touches more then one, so stop sliding and allow browser do default action
				if(touches.length > 1) {
					return false;
				}
				point = touches[0];	
				e.preventDefault();				
			} else {
				point = e;
				e.preventDefault();		
			}
			moveTo(point);

			mouseMoveHandlerTooltip(point);
			
			return false;		
		}
		
		function onDragRelease(e) {
			if(isPressed){	
				isPressed = false;			
				_doc.off(events.moveEvent).off(events.upEvent);	
				
				var point;
				if(e.type == 'touchend'){
					var touches;
					if(e.originalEvent.touches && e.originalEvent.touches.length) {
						touches = e.originalEvent.touches;
					}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
						touches = e.originalEvent.changedTouches;
					}else{
						return false;
					}
					// If touches more then one, so stop sliding and allow browser do default action
					if(touches.length > 1) {
						return false;
					}
					point = touches[0];	
					e.preventDefault();				
				} else {
					point = e;
					e.preventDefault();		
				}
				moveTo(point);

				tooltip.hide();
			}
			
			return false;
		}	

		function moveTo(v) {
			var value;
			if(isVertical){
				value = Math.max(0, Math.min(1, (v.pageY - sliderBg.offset().top) / rangeSize));
				value = 1 - value;//reverse for up dir
			}else{
				value = Math.max(0, Math.min(1, (v.pageX - sliderBg.offset().left) / rangeSize));
			} 
			self.setValue(value);
		}

		this.setValue = function(percent){

			if(!HAPUtils.isNumber(rangeSize))rangeSize = isVertical ? sliderBg.height() : sliderBg.width();
			sliderLevel.css(prop, percent*rangeSize+'px');

			//convert to playback rate
			var val = min+(max-min)*percent;
			val = Math.round( val * 10) / 10;//one decimal

			$(self).trigger("HAPPlaybackRateSlider.RANGE_CHANGE", {value:val});

		}

		this.setVisual = function(v){

			//convert to percent
			var percent = (v - min) / (max-min);

			if(!HAPUtils.isNumber(rangeSize))rangeSize = isVertical ? sliderBg.height() : sliderBg.width();
			sliderLevel.css(prop, percent*rangeSize+'px');

		}

		//############################################//
		/* tooltip */
		//############################################//

		if(!HAPUtils.isMobile()){
			seekbar.on('mouseover', function mouseOverHandlerSeek(){
				if(isPressed)return;
				seekbar.on(events.moveEvent, mouseMoveHandlerTooltip).on('mouseout', mouseOutHandler);
				_doc.on('mouseout', mouseOutHandler);
			});
			function mouseOutHandler() {
				seekbar.off(events.moveEvent, mouseMoveHandlerTooltip).off('mouseout', mouseOutHandler);
				_doc.off('mouseout', mouseOutHandler);
				tooltip.hide();
			}
		}
			
		function mouseMoveHandlerTooltip(e){

			if(!isVertical)var s = e.pageX - sliderBg.offset().left;
			else var s = e.pageY - sliderBg.offset().top;
			if(s < 0) s = 0;
			else if(s > rangeSize) s = rangeSize;
			var newPercent = Math.max(0, Math.min(1, s / rangeSize));
			if(!HAPUtils.isNumber(newPercent))return false;
			if(isVertical)newPercent = 1 - newPercent;//reverse for up dir
			
			var val = min+(max-min)*newPercent;
			val = Math.round( val * 10) / 10;
			//console.log(val)

			tooltip.text(val);

			var main = wrapper[0].getBoundingClientRect(),
			element = seekbar[0].getBoundingClientRect();

			if(isVertical){
				var l = parseInt(element.left - main.left - tooltip.outerWidth()/2 + seekbar.outerWidth()/2),
					t = parseInt(e.pageY - _window.scrollTop() - main.top - tooltip.outerHeight() - 10);

				//restrain
				if(t < element.top - main.top - tooltip.outerHeight() - 10)t = element.top - main.top - tooltip.outerHeight() - 10;
				else if(t > element.top - main.top + seekbar.outerHeight() - tooltip.outerHeight()) t = element.top - main.top + seekbar.outerHeight() - tooltip.outerHeight();

			}else{
				var l = parseInt(e.pageX - _window.scrollLeft() - main.left - tooltip.outerWidth()/2),
					t = parseInt(element.top - main.top - tooltip.outerHeight());

				//restrain
				if(l < element.left - main.left)l = element.left - main.left;
				else if(l > element.left - main.left + seekbar.outerWidth() - tooltip.outerWidth()) l = element.left - main.left + seekbar.outerWidth() - tooltip.outerWidth();
			}

			//keep within component
			/*if(l + tooltip.outerWidth() > wrapper.width())l = wrapper.width() - tooltip.outerWidth();
			else if(l < 0) l = 0;*/

			if(t + main.top < 0)t = parseInt(element.top - main.top + tooltip.outerHeight() + 15);//position below
			if(t < 0) t = 0;
			
			tooltip.css({
				left: l+'px',
				top: t+'px'
			}).show();
		}

	}

	window.HAPPlaybackRateSlider = HAPPlaybackRateSlider;

}(window,jQuery));


(function (window, $){
	"use strict"	
	var HAPRangeSlider = function (data){

		var self = this,
		_doc = $(document),
		_window = $(window),
		isPressed,
		events = HAPUtils.getEvents(),
		range_handle_a = data.range_handle_a,
		range_handle_b = data.range_handle_b,
		elem


		range_handle_a.on(events.downEvent,function(e){
			elem = $(this).css('z-index',1);//make last dragged z index higher, so we cannot drag into corner one over another
			range_handle_b.css('z-index',0);
			onDragStart(e);
			return false;		
		});

		range_handle_b.on(events.downEvent,function(e){
			elem = $(this).css('z-index',1);
			range_handle_a.css('z-index',0);
			onDragStart(e);
			return false;		
		});

		function onDragStart(e) {
			if(!isPressed){					
				var point;
				if(e.type == 'touchstart'){
					var currTouches = e.originalEvent.touches;
					if(currTouches && currTouches.length > 0) {
						point = currTouches[0];
					}else{	
						return false;						
					}
				}else{
					point = e;								
					e.preventDefault();						
				}
				isPressed = true;
				_doc.on(events.moveEvent, function(e) { onDragMove(e); }).on(events.upEvent, function(e) { onDragRelease(e); });		
			}
			return false;	
		}
					
		function onDragMove(e) {	
			var point;
			if(e.type == 'touchmove'){
				var touches;
				if(e.originalEvent.touches && e.originalEvent.touches.length) {
					touches = e.originalEvent.touches;
				}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
					touches = e.originalEvent.changedTouches;
				}else{
					return false;
				}
				// If touches more then one, so stop sliding and allow browser do default action
				if(touches.length > 1) {
					return false;
				}
				point = touches[0];	
				e.preventDefault();				
			} else {
				point = e;
				e.preventDefault();		
			}
			
			$(self).trigger("HAPRangeSlider.RANGE_CHANGE", {point:point, elem: elem, event:point});
			
			return false;		
		}
		
		function onDragRelease(e) {
			if(isPressed){	
				isPressed = false;			
				_doc.off(events.moveEvent).off(events.upEvent);	
				
				var point;
				if(e.type == 'touchend'){
					var touches;
					if(e.originalEvent.touches && e.originalEvent.touches.length) {
						touches = e.originalEvent.touches;
					}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
						touches = e.originalEvent.changedTouches;
					}else{
						return false;
					}
					// If touches more then one, so stop sliding and allow browser do default action
					if(touches.length > 1) {
						return false;
					}
					point = touches[0];	
					e.preventDefault();				
				} else {
					point = e;
					e.preventDefault();		
				}
				
				$(self).trigger("HAPRangeSlider.RANGE_CHANGE", {point:point, elem: elem});
			}
			
			return false;
		}

		this.isDrag = function(){
			return isPressed;
		}	

	}

	window.HAPRangeSlider = HAPRangeSlider;

}(window,jQuery));



var hapjq=jQuery;
(function($) {

	$.fn.hap = function(settings) {

	"use strict"



	//############################################//
	/* settings */
	//############################################//

	var defaults = {
		sourcePath:"",
		playlistItemContent:"",
		statisticsContent:"plays,likes,downloads",
		audioFormatOrder:'aac,wav,mp3',
		dataInterval: 250,
		volume:0.5,
		activeItem:0,
		numberTitleSeparator:".&nbsp;",
		artistTitleSeparator:"&nbsp;-&nbsp;",
		useNumbersInPlaylist:false,
		preload:"auto",
		autoPlay:false,
		loopState:"playlist",
		randomPlay:false,
		facebookAppId:"",
		usePlaylistScroll:false,
		useKeyboardNavigationForPlayback:false,
		togglePlaybackOnMultipleInstances:true,
		useTitleScroll:false,
		titleScrollSpeed: 1,
        titleScrollSeparator: " *** ",
        continousKey:"hap-continous-key",
        useMediaSession:true,
        createDownloadIconsInPlaylist: false,
        createLinkIconsInPlaylist: false,
        downloadIconTitle:"Download",
        tooltipStatPlays: "Plays",
        tooltipStatLikes: "Likes",
        tooltipStatDownloads: "Downloads",
        artworkSize: [100,200,250,340,460,600],
        soundCloudThumbQuality: "t300x300.jpg",
        playbackRateMin: 0.5, 
		playbackRateMax: 2,
		sortOrder: "",
		searchDescriptionInPlaylist:true,
		hideYoutubeAfterStart:false,
		getRadioArtwork:true,
		defaultSongArtist: "DATA NOT AVAILABLE",
        defaultSongTitle: "DATA NOT AVAILABLE",
        lastPlayedInterval: 10000,
        enableCors: true,
        cors: 'https://kastproxy-us.herokuapp.com/,https://kastproxy-eu.herokuapp.com/,https://cors-anywhere.herokuapp.com/,https://cors.io/?',
        useCorsForAudio:false,

        whatsAppWarning:"Please share this content on mobile device!",
		downloadIcon: "<svg viewBox='0 0 512 512'><path d='M216 0h80c13.3 0 24 10.7 24 24v168h87.7c17.8 0 26.7 21.5 14.1 34.1L269.7 378.3c-7.5 7.5-19.8 7.5-27.3 0L90.1 226.1c-12.6-12.6-3.7-34.1 14.1-34.1H192V24c0-13.3 10.7-24 24-24zm296 376v112c0 13.3-10.7 24-24 24H24c-13.3 0-24-10.7-24-24V376c0-13.3 10.7-24 24-24h146.7l49 49c20.1 20.1 52.5 20.1 72.6 0l49-49H488c13.3 0 24 10.7 24 24zm-124 88c0-11-9-20-20-20s-20 9-20 20 9 20 20 20 20-9 20-20zm64 0c0-11-9-20-20-20s-20 9-20 20 9 20 20 20 20-9 20-20z'></path></svg>",

		linkIcon: "<svg viewBox='0 0 576 512'><path d='M504.717 320H211.572l6.545 32h268.418c15.401 0 26.816 14.301 23.403 29.319l-5.517 24.276C523.112 414.668 536 433.828 536 456c0 31.202-25.519 56.444-56.824 55.994-29.823-.429-54.35-24.631-55.155-54.447-.44-16.287 6.085-31.049 16.803-41.548H231.176C241.553 426.165 248 440.326 248 456c0 31.813-26.528 57.431-58.67 55.938-28.54-1.325-51.751-24.385-53.251-52.917-1.158-22.034 10.436-41.455 28.051-51.586L93.883 64H24C10.745 64 0 53.255 0 40V24C0 10.745 10.745 0 24 0h102.529c11.401 0 21.228 8.021 23.513 19.19L159.208 64H551.99c15.401 0 26.816 14.301 23.403 29.319l-47.273 208C525.637 312.246 515.923 320 504.717 320zM408 168h-48v-40c0-8.837-7.163-16-16-16h-16c-8.837 0-16 7.163-16 16v40h-48c-8.837 0-16 7.163-16 16v16c0 8.837 7.163 16 16 16h48v40c0 8.837 7.163 16 16 16h16c8.837 0 16-7.163 16-16v-40h48c8.837 0 16-7.163 16-16v-16c0-8.837-7.163-16-16-16z'></path></svg>",

		statDownloadIcon: "<svg viewBox='0 0 512 512'><path d='M504 256c0 137-111 248-248 248S8 393 8 256 119 8 256 8s248 111 248 248zM212 140v116h-70.9c-10.7 0-16.1 13-8.5 20.5l114.9 114.3c4.7 4.7 12.2 4.7 16.9 0l114.9-114.3c7.6-7.6 2.2-20.5-8.5-20.5H300V140c0-6.6-5.4-12-12-12h-64c-6.6 0-12 5.4-12 12z'></path></svg>",

		statLikeIcon: "<svg viewBox='0 0 512 512'><path d='M462.3 62.6C407.5 15.9 326 24.3 275.7 76.2L256 96.5l-19.7-20.3C186.1 24.3 104.5 15.9 49.7 62.6c-62.8 53.6-66.1 149.8-9.9 207.9l193.5 199.8c12.5 12.9 32.8 12.9 45.3 0l193.5-199.8c56.3-58.1 53-154.3-9.8-207.9z'></path></svg>",

		statPlayIcon: "<svg viewBox='0 0 512 512'><path d='M256 8C119 8 8 119 8 256s111 248 248 248 248-111 248-248S393 8 256 8zm115.7 272l-176 101c-15.8 8.8-35.7-2.5-35.7-21V152c0-18.4 19.8-29.8 35.7-21l176 107c16.4 9.2 16.4 32.9 0 42z'></path></svg>",

	}

	var settings = $.extend(true, {}, defaults, settings);




	//############################################//
	/* url params */
	//############################################//

	//check url params, overwrite settings
	//check playlist from url params

	var urlParams = HAPUtils.getUrlParameter(),
	queryPlaylistArr = [], 
	queryPlaylist = [];

	Object.keys(urlParams).forEach(function(key) {
		if(key.indexOf('hap-')==0){//url params start with 'hap-'
			var k = key.substr(4),//remove hap-
			camelCased = k.replace(/-([a-z])/g, function (g) { return g[1].toUpperCase(); });
			var value = decodeURIComponent(urlParams[key]).replace(/\+/g, ' ');//+ to space
			if(value){
				if(settings.hasOwnProperty(camelCased)){//update settings 
					if(value === "true")value = true;
					else if(value === "false")value = false;
					settings[camelCased] = value;//only accept if property already exist
				}else{//check for playlist
					value = value.split(",").map(function(item){
					    return item.trim();
					});
					queryPlaylistArr.push({key:camelCased, value: value})
				}
			}
		}
	});
	//playlist from url params
	if(queryPlaylistArr.length){
		var i, j, len = queryPlaylistArr.length, len2 = queryPlaylistArr[0].value.length, item, obj, value;
		for(j = 0;j<len2;j++){
			obj = {};
			for(i = 0;i<len;i++){
				item = queryPlaylistArr[i];
				value = item.value[j];
				if(value === "true")value = true;
				else if(value === "false")value = false;
				obj[item.key] = value;
				if(item.key === 'type')obj['origtype'] = value;//for stats
			}
			queryPlaylist.push(obj);
		}
	}

	urlParams = null;


	//############################################//
	/* elements */
	//############################################//

	var wrapper = $(this);

	var class_list = '';
	if(wrapper.attr('class') != undefined){
		class_list = wrapper.attr('class').split(/\s+/);
		if(class_list.indexOf('hap-fixed')>-1)wrapper.appendTo($('body'));
	}
	

	if(settings.markup)	wrapper.append(settings.markup);

	var playlistList = $(settings.playlistList),
	playlistHolder = wrapper.find('.hap-playlist-holder'),
	playlistInner = wrapper.find('.hap-playlist-inner'),
	playlistContent = wrapper.find('.hap-playlist-content'),
	playlistFilterMsg = wrapper.find('.hap-playlist-filter-msg'),
	sortAlpha = wrapper.find('.hap-sort-alpha'),
	popupToggle = wrapper.find('.hap-popup-toggle'),
	progressBg = wrapper.find('.hap-progress-bg'),
	loadLevel = wrapper.find('.hap-load-level'),
	progressLevel = wrapper.find('.hap-progress-level'),
	volumeToggle = wrapper.find('.hap-volume-toggle'),
	volumeSeekbar = wrapper.find('.hap-volume-seekbar'),
	volumeBg = wrapper.find('.hap-volume-bg'),
	volumeLevel = wrapper.find('.hap-volume-level'),
	playerHolder = wrapper.find('.hap-player-holder'),
	playerThumb = wrapper.find('.hap-player-thumb').attr('role', 'img'),
	shareHolder = wrapper.find('.hap-share-holder'),
	playbackRateHolder = wrapper.find('.hap-playback-rate-holder'),
	playerTitle = wrapper.find('.hap-player-title'),
	playerArtist = wrapper.find('.hap-player-artist'),
	mediaTimeCurrent = wrapper.find('.hap-media-time-current'),
	mediaTimeTotal = wrapper.find('.hap-media-time-total'),
	mediaTimeAd = wrapper.find('.hap-media-time-ad'),
	playbackToggle = wrapper.find('.hap-playback-toggle'),
	loopToggle = wrapper.find('.hap-loop-toggle'),
	randomToggle = wrapper.find('.hap-random-toggle'),
	rangeToggle = wrapper.find('.hap-range-toggle'),
	playbackRateToggle = wrapper.find('.hap-playback-rate-toggle'),
	shareToggle = wrapper.find('.hap-share-toggle'),
	preloader = wrapper.find('.hap-preloader');

	
	
	//############################################//
	/* vars */
	//############################################//
	
	var isMobile = HAPUtils.isMobile(),
	hasLocalStorage = HAPUtils.hasLocalStorage(),

	sourcePath = settings.sourcePath,
	instanceName = settings.instanceName,
	activePlaylist = settings.activePlaylist,
	preload = settings.preload,
	autoPlay = settings.autoPlay,
	loopState = settings.loopState,
	randomPlay = settings.randomPlay,
	useNumbersInPlaylist = settings.useNumbersInPlaylist,
	usePlaylistScroll = settings.usePlaylistScroll,
	playlistScrollOrientation = settings.playlistScrollOrientation,
	facebookAppId = settings.facebookAppId,
	soundCloudAppId = settings.soundCloudAppId,
	gDriveAppId = settings.gDriveAppId,
	volume = settings.volume,
	useKeyboardNavigationForPlayback = settings.useKeyboardNavigationForPlayback,
    playlistItemContent = settings.playlistItemContent.replace(/\s+/g, '').split(','),
    statisticsContent = settings.statisticsContent.replace(/\s+/g, '').split(','),
    useContinousPlayback = settings.useContinousPlayback && hasLocalStorage,
    continousPlaybackTrackAllSongs = useContinousPlayback && settings.continousPlaybackTrackAllSongs,
    continousKey = settings.continousKey,
    useMediaSession = settings.useMediaSession && 'mediaSession' in navigator && window.location.protocol != 'file:',
    numberTitleSeparator = settings.numberTitleSeparator,
	artistTitleSeparator = settings.artistTitleSeparator,
	downloadIconTitle = settings.downloadIconTitle,
	useTitleScroll = settings.useTitleScroll,
	useStatistics = settings.useStatistics,
	tooltipStatPlays = settings.tooltipStatPlays,
	tooltipStatLikes = settings.tooltipStatLikes,
	tooltipStatDownloads = settings.tooltipStatDownloads,
 	useGa = settings.useGa && settings.gaTrackingId,
 	downloadIcon = settings.downloadIcon,
	linkIcon = settings.linkIcon,
	statDownloadIcon = settings.statDownloadIcon,
	statLikeIcon = settings.statLikeIcon,
	statPlayIcon = settings.statPlayIcon,
	playlistOpened = settings.playlistOpened,
	sortOrder = settings.sortOrder,
	searchDescriptionInPlaylist = settings.searchDescriptionInPlaylist,
	hideYoutubeAfterStart = settings.hideYoutubeAfterStart,
	audioFormatOrder = settings.audioFormatOrder.split(",").map(function(item){
	    return item.trim();
	}),
	createDownloadIconsInPlaylist = settings.createDownloadIconsInPlaylist,
    createLinkIconsInPlaylist = settings.createLinkIconsInPlaylist,
    useCorsForAudio = settings.useCorsForAudio,
    disableSongSkip = settings.disableSongSkip,
    pauseAudioDuringAds = settings.pauseAudioDuringAds,
    fetchPlayerArtwork = settings.fetchPlayerArtwork && playerThumb.length,
    allowOnlyOneOpenedAccordion = settings.allowOnlyOneOpenedAccordion

    if(settings.playlistElement)playlistContent = $(settings.playlistElement)//custom playlist element









	var self = this,
	componentInited,
	playlistTransitionOn,
	_body = $('body'),
	_window = $(window),
	_doc = $(document),
	windowResizeTimeoutID,
	windowResizeTimeout = 150,
	hasDownloadSupport = HAPUtils.hasDownloadSupport(),
	isIOS = HAPUtils.isIOS(),
	isAndroid = HAPUtils.isAndroid(),
	mp3Support = HAPUtils.canPlayMp3(),
	wavSupport = HAPUtils.canPlayWav(),
	aacSupport = HAPUtils.canPlayAac(),
	isChrome = HAPUtils.isChrome(),
	isSafari = HAPUtils.isSafari(),
	volumeCanBeSet = HAPUtils.volumeCanBeSet(),
	proxy = sourcePath + 'includes/ba-simple-proxy.php', 
	bsf_match = 'ebsfm:',
	loopStateArr = ['playlist','single','off'],
	loopStateCounter = loopStateArr.indexOf(loopState),
	nonProcessMedia = ['audio', 'hls', 'shoutcast', 'icecast', 'radiojar'],
	mediaPrefixUrl,

	//masonry
	_masonry,
	
	//audio
	id3Counter,
	id3Start,
	audio = $(document.createElement("audio")).attr('preload',preload),
	audioUp2Js = audio[0],
	seekTime = 10,//seconds
	startEventDone,//lot of delay on ios

	//add more on total scroll
	addMoreOnTotalScroll,
	addMoreLimit,
	addMoreNumResults,
	addMoreOffset,
	encryptMediaPaths,
	addMoreSortOrder,//asc, desc

	//load more
	nextPageToken,
	loadMoreProcess,
	loadMoreType,
 	loadMoreOnTotalScroll,
 	useId3,
 	allResults = 999999999,

	//soundcloud
	soundCloudInited,
	resultLimit,
	soundCloudResultArr = [],

	//hls
	hls,
	hlsSupport,
	hlsInited,

	//radio
	_HAPRadioData,

	//ads
	_HAPAdManager,
	iosAdMidFixStart, 

	//share
	_HAPShareManager,

	//youtube
	_HAPYoutubeLoader,
	youtubeHolder,
	youtubePlayer,
	youtubeInited,
	youtubeReady,
	youtubeStarted,
	youtubeEndFix,//end event fires multiple with end seconds?
	youtubeBlocker,
	youtubePlayed,//add autoplay after this 

	seekbarDisabled,
	playlistScrollInit,
	playlistClick,//scroll to active playlist item
	usePlaylist = true,

	//global playlist otions
	playerThumbInited,
	globalThumb,
	globalStartTime,
	globalEndTime,
	globalPlaybackRate,
	globalAdPre,
	globalAdMid,
	globalAdMidInterval,
	globalAdEnd,

	//addTrack helpers
	endInsert,//insert at playlist end 
	insertPosition, 
	addTrackProcess,
	addTrackPlayit,
	playlistFirstInit,//first time create playlist with active item 
	
	playlistProcessCounter = -1,
	playlistDataArr = [],
	playlistProcessData = [],
	playlistProcessDataUrl = [],
	playlistData,
	playlistLength=0,
	autoInitActiveItem,
	lastPlaylist,
	activePlaylistItem,
	mediaStarted,
	mediaType,
	mediaTitle,
	mediaPath,
	mediaCounter,
	mediaPlaying = false,
	currMediaData,
	dataInterval = settings.dataInterval,
	dataIntervalID, 
	seekBarDown, 
	seekBarSize,
	events = HAPUtils.getEvents(), 
	playlistId,//used for statistics or add more

	_HAPPlaybackRateSlider,
	_HAPRangeSlider,

	hoverClass = "hap-contr-btn-hover",

	//scripts
	hls_js = "https://cdn.jsdelivr.net/npm/hls.js@latest",
	youtube_js = "https://www.youtube.com/iframe_api",

	//accordion
	lastOpenedAccordion,
    accordionLoading,
	accordionMode,
	accordionHolder,//playlistContent for accordion 
	accordionInited,
	accordionActiveId,//active accordion item
	accordioPlaylistId,//currently playing accordion playlist
	playlistDataAccordion = [],//save playlist data for each accordion item
	accordionUseId3//use id3 tags in accordion

	if(accordionMode)usePlaylist = true


	//############################################//
	/* setup */
	//############################################//



	if(playlistItemContent.indexOf('description') == -1)searchDescriptionInPlaylist = false;


	if(typeof window.hap_mediaArr === 'undefined')window.hap_mediaArr = [];
	window.hap_mediaArr.push({inst: self, id: instanceName});//save instance so we can pause playback in one instance if other is started

	if(isMobile){
		autoPlay = false;
	}


	//loop 
	if(loopStateCounter == -1)loopState = 'off';
	loopStateCounter = loopStateArr.indexOf(loopState);
	loopToggle.find('.hap-btn-loop-'+loopState).show();
	//shuffle
	if(randomPlay)randomToggle.find('.hap-btn-random-on').show();
	else randomToggle.find('.hap-btn-random-off').show();







    //hidden playlist from dom 
    if(playlistContent.length == 0){
    	playlistContent = $('<div style="display:none;"></div>');
    	usePlaylist = false;
    	usePlaylistScroll = false;
    	useNumbersInPlaylist = false;
    }

    //set icon visibility on start
	volumeToggle.find('.hap-btn-volume-up').show();
	sortAlpha.find('.hap-btn-sort-alpha-down').show();
	playbackToggle.find('.hap-btn-play').show();
    
	



	//############################################//
	/* load more */
	//############################################//

    var loadMoreBtn = wrapper.find('.hap-load-more-btn').on('click',function(){
    	if(!componentInited) return false;
    	if(loadMoreProcess) return false;
    	if(loadMoreOnTotalScroll)self.loadMore();
    	else if(addMoreOnTotalScroll){
			addMore();
    	}
    	loadMoreBtn.css('opacity',0);
    });



    //############################################//
	/* title scroll */
	//############################################//

	var _HAPTextScroller;

	if(useTitleScroll){

		if(typeof HAPTextScroller === 'undefined'){
			alert('link to textscroller.js file missing in head tag');
		}else{
			_HAPTextScroller = new HAPTextScroller({settings:settings, mask:wrapper.find('.hap-media-title-mask'), container:wrapper.find('.hap-media-title')});
		}
		
	}



	//############################################//
	/* filter tracks */
	//############################################//

	var searchFilter = wrapper.find('.hap-search-filter').on('keyup.apfilter',function(){
		if(playlistLength == 0)return false;

		var value = $(this).val().toLowerCase(), i, j = 0, title, pi;

		for(i = 0; i< playlistLength; i++){

			pi = playlistContent.children('.hap-playlist-item').eq(i);

			title = pi.find('.hap-playlist-title').html().toLowerCase();

			if(searchDescriptionInPlaylist && pi.find('.hap-playlist-description').length){
				title += pi.find('.hap-playlist-description').html().toLowerCase();
			}

			if(title.indexOf(value) >- 1){
				pi.show();
			}else{
				pi.hide();
				j++;
			}
		}
	
		if(j == playlistLength)playlistFilterMsg.show();
		else playlistFilterMsg.hide();

	});


	//############################################//
	/* share */
	//############################################//
	
	if(!_HAPShareManager){

		if(typeof HAPShareManager === 'undefined'){
			alert('link to sharemanager.js file missing in head tag!');
		}

	  	_HAPShareManager = new HAPShareManager(settings);
	  
	}

	//############################################//
	/* playlist manager */
	//############################################//
	
	var _playlistManager = new HAPPlaylistManager({loop:loopState, random:randomPlay});
	$(_playlistManager).on('HAPPlaylistManager.COUNTER_READY', function(e, counter){
		if(mediaType)cleanMedia();

		mediaCounter = counter;
		currMediaData = playlistDataArr[mediaCounter].data;
		//console.log(currMediaData,playlistDataArr)

		mediaType = currMediaData.type;
		if(globalThumb)currMediaData.thumb = globalThumb;
		if(globalPlaybackRate)currMediaData.playbackRate = globalPlaybackRate;
		if(globalStartTime)currMediaData.start = globalStartTime;
		if(globalEndTime)currMediaData.end = globalEndTime;
		//individual ads will overwrite global ads
		if(globalAdPre && !currMediaData.adPre)currMediaData.adPre = globalAdPre;
		if(globalAdMid && !currMediaData.adMid)currMediaData.adMid = globalAdMid;
		if(globalAdMidInterval)currMediaData.adMidInterval = globalAdMidInterval;
		if(globalAdEnd && !currMediaData.adEnd)currMediaData.adEnd = globalAdEnd;


		//toggle non used element
		if(mediaType == 'shoutcast' || mediaType == 'icecast' || mediaType == 'radiojar' || mediaType == 'hls'){
			rangeToggle.hide();//cant use range for radio
			seekbar.hide();//cant seek
			mediaTimeCurrent.hide();
			mediaTimeTotal.hide();

			if(mediaType == 'hls')setTitleAndThumb();

		}else{
			rangeToggle.show();
			seekbar.show();
			mediaTimeCurrent.show();
			mediaTimeTotal.show();

			setTitleAndThumb();
			if(useMediaSession)updateMetadata();
		}

	

		//check last position
		if(continousPlaybackTrackAllSongs){
			if(settings.lastPositionArr){
				var i, len = settings.lastPositionArr.length, item;
				for(i = 0;i<len; i++){
					item = settings.lastPositionArr[i];
					if((item.title && currMediaData.title == item.title) && currMediaData.artist == item.artist){
						currMediaData.start = item.start;
						break;
					}
				}
			}
		}

		disableActivePlaylistItem(mediaCounter);

	
		if((!currMediaData.thumb && !currMediaData.thumbDefault) && fetchPlayerArtwork)fetchArtwork();//get artwork manually


		//############################################//
		/* ads */
		//############################################//

		if(currMediaData.adPre || currMediaData.adMid || currMediaData.adEnd){
			if(!_HAPAdManager){

				if(typeof HAPAdManager === 'undefined'){
					alert('link to admanager.js file missing in head tag!');
				}

		      	_HAPAdManager = new HAPAdManager(settings, self, currMediaData, volume, audioUp2Js, progressLevel, mediaTimeAd, seekBarSize);
		      	$(_HAPAdManager).on('HAPAdManager.IOS_ADMID_FIX_START', function(e,data){

		      		iosAdMidFixStart = true;

				}).on('HAPAdManager.ADPRE_PLAY', function(e,data){

		      		progressLevel.addClass('hap-ad-progress-level');
					mediaTimeCurrent.hide();
					mediaTimeTotal.hide();
					mediaTimeAd.show();

				}).on('HAPAdManager.ADPRE_ENDED', function(e,data){

		      		progressLevel.removeClass('hap-ad-progress-level');
			    	mediaTimeAd.hide();
			    	if(mediaType != 'shoutcast' && mediaType != 'icecast' && mediaType != 'radiojar'){
				    	mediaTimeCurrent.show();
						mediaTimeTotal.show();
					}

			    	if(mediaType == 'youtube'){
						initYoutube();
					}else{
						initAudio();
					}

				}).on('HAPAdManager.ADMID_PLAY', function(e,data){
		      			
					self.pauseMedia();

					progressLevel.addClass('hap-ad-progress-level');
					mediaTimeCurrent.hide();
					mediaTimeTotal.hide();
					mediaTimeAd.show();

				}).on('HAPAdManager.ADMID_ENDED', function(e,data){
		      			
					progressLevel.removeClass('hap-ad-progress-level');
			    	mediaTimeAd.hide();
			    	if(mediaType != 'shoutcast' && mediaType != 'icecast' && mediaType != 'radiojar'){
				    	mediaTimeCurrent.show();
						mediaTimeTotal.show();
					}

					self.playMedia();

				}).on('HAPAdManager.ADEND_PLAY', function(e,data){
		      			
					progressLevel.addClass('hap-ad-progress-level');
					mediaTimeCurrent.hide();
					mediaTimeTotal.hide();
					mediaTimeAd.show();

				}).on('HAPAdManager.ADEND_ENDED', function(e,data){
		      			
					progressLevel.removeClass('hap-ad-progress-level');
			    	mediaTimeAd.hide();
			    	if(mediaType != 'shoutcast' && mediaType != 'icecast' && mediaType != 'radiojar'){
				    	mediaTimeCurrent.show();
						mediaTimeTotal.hide();
					}

					audioEndedAction(true);

				});

		    }else{
		    	_HAPAdManager.setAdData(currMediaData);
		    }
		}


		if(currMediaData.adPre){

			_HAPAdManager.initAdPre();

			autoPlay = true;
			audioUp2Js.volume = volume;

			var promise = audioUp2Js.play();
			if (promise !== undefined) {
				promise.then(function(){
					//manually put play icon (since there is no playHandler) so it doesnt confuse the user when he pressed play and no pause icon appeard 
					playbackToggle.find('.hap-btn-play').hide();
					playbackToggle.find('.hap-btn-pause').show();
			    }).catch(function(error){
				});
			}

		//############################################//
		/* ads */
		//############################################//

		}else{
			if(mediaType == 'youtube'){
				initYoutube();
			}else{
				initAudio();
			}
		}

	})
	.on('HAPPlaylistManager.PLAYLIST_END', function(){
		$(self).trigger("playlistEnd", {instance:self, instanceName:instanceName});
	});

	

	//############################################//
	/* youtube player */
	//############################################//

	function initYoutube() {

		mediaPath = currMediaData.mp3;
		if(mediaPath.indexOf(bsf_match) != -1)mediaPath = atob(mediaPath.substr(6));

		if(!youtubeInited){

			if(playerThumb.length == 0){
				alert('Using Youtube requires player to have thumbnail image in player where Youtube player will be placed. Please use demo which has thumbnail in player! hap-player-thumb element');
				return;
			}

			youtubeHolder = $('<div class="hap-youtube-holder"/>').appendTo(playerThumb);

			var yt_frameId = 'ytplayer' + Math.floor(Math.random()*0xFFFFFF),
				prefix = ('https:' == window.location.protocol ? 'https:' : 'http:'),
				yt_embed = '//www.youtube-nocookie.com/embed/',
				enablejsapi = '?enablejsapi=1',
				controls = '&controls=0',
				wmode = '&wmode=transparent',
				showinfo = '&showinfo=0',
				modestbranding = '&modestbranding=1',
				rel = '&rel=0',
				autoplay = '&autoplay=1',
				playsinline = '&playsinline=1',
				iv_load_policy = '&iv_load_policy=3',
				cc_load_policy = '&cc_load_policy=0',
				domain = window.location.href.split("/"),
				orig = domain[0] + "//" + domain[2],
				origin = '&origin=' + orig;

			var iframeSrc = prefix + yt_embed + mediaPath + enablejsapi + controls + rel + showinfo + playsinline + modestbranding + wmode + iv_load_policy + cc_load_policy;

			if(/^http/.test(orig))iframeSrc += origin;

			if(settings.resumeTime){
				iframeSrc += '&start='+settings.resumeTime;
				delete settings.resumeTime;
			}else if(currMediaData.start)iframeSrc += '&start='+currMediaData.start;
			if(currMediaData.end)iframeSrc += '&end='+currMediaData.end;

			var youtubeIframe = $('<iframe/>',{
				id: yt_frameId,
				frameborder: 0,
				src: iframeSrc,
				width: '100%',
				height: '100%',
				webkitAllowFullScreen: false,
				mozallowfullscreen: false,
				allowFullScreen: false,
				allow: "accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture"
			}).addClass('hap-media');

			youtubeHolder.show().prepend(youtubeIframe); 

			if(!window.YT){
				var tag = document.createElement('script');
				tag.src = youtube_js;
				var firstScriptTag = document.getElementsByTagName('script')[0];
				firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
			}

			var interval = setInterval(function(){
				if(window.YT && window.YT.Player){
					if(interval) clearInterval(interval);

					youtubePlayer = new YT.Player(yt_frameId, {
						events: {
							'onReady': onYtPlayerReady,
							'onStateChange': onYtPlayerStateChange,
							'onError': onYtPlayerError
						}
					});
				}
			}, 100);
		 
			youtubeInited = true;

		}else{

			if(youtubeReady){

				var st = 0;
				if(settings.resumeTime)st = resumeTime;
				else if(currMediaData.start)st = currMediaData.start;

				if(autoPlay){
					youtubePlayer.loadVideoById({'videoId':mediaPath, 'startSeconds':st, 'endSeconds':currMediaData.end, 'suggestedQuality':currMediaData.quality});
				}else{
					youtubePlayer.cueVideoById({'videoId':mediaPath, 'startSeconds':st, 'endSeconds':currMediaData.end, 'suggestedQuality':currMediaData.quality});
				}
				
			}
		}

		youtubeHolder.show();
	}

	window.onYouTubeIframeAPIReady = function() {}

	function onYtPlayerReady(event) {
		youtubeReady = true;
		
		if(youtubePlayed && autoPlay){
			youtubePlayer.playVideo();
		}
	}

	function onYtPlayerStateChange(event) {
		if(!youtubeHolder.is(":visible"))return;
	
		if(event.data == -1){//unstarted
		}
		else if(event.data == 0){//ended

			if(useRange && rangeOn && !rangeSet && !_HAPRangeSlider.isDrag()){
				rangeSet = true;
				youtubePlayer.seekTo(range_a_time);
				youtubePlayer.playVideo();
				return;
			}

			if(youtubeEndFix)return;
			youtubeEndFix = true;
			mediaEndHandler();
		}
		else if(event.data == 1){//playing
			
			if(!youtubeStarted){
				youtubeEndFix = false;

				youtubePlayer.setPlaybackRate(currMediaData.playbackRate);
				if(_HAPPlaybackRateSlider)_HAPPlaybackRateSlider.setVisual(currMediaData.playbackRate);

				if(!youtubeBlocker){
					youtubeBlocker =  $('<div class="hap-iframe-blocker"></div>').css('display','block').appendTo(youtubeHolder);
				}

				youtubeStarted = true;	
				autoPlay = true;
				youtubePlayed = true;

				if(hideYoutubeAfterStart)youtubeHolder.css('opacity',0);
			}

			if(useRange && rangeOn){
				setTimeout(function(){
					clearTimeout(this);
					rangeSet = false;
				},1000);
			}

			if(dataIntervalID) clearInterval(dataIntervalID);
			dataIntervalID = setInterval(trackProgress, dataInterval);
			
			playHandler();

		}
		else if(event.data == 2){//paused
			pauseHandler();
		}
		else if(event.data == 3){//buffering
			
		}
		else if(event.data == 5){//cued
		
		}
	}
	
	function onYtPlayerError(e) {
		var error = 'Youtube erorr';
		switch(e.data){
			case 2:
			error = "Error code = "+e.data+": The request contains an invalid parameter value. For example, this error occurs if you specify a video ID that does not have 11 characters, or if the video ID contains invalid characters, such as exclamation points or asterisks."
			break;
			case 100:
			error = "Error code = "+e.data+": Video not found, removed, or marked as private"
			break;
			case 101:
			error = "Error code = "+e.data+": Embedding disabled for this video"
			break;
			case 150:
			error = "Error code = "+e.data+": Video not found, removed, or marked as private [same as error 100]"
			break;
		}

		$(self).trigger('soundError', {instance:self, instanceName:instanceName, data:currMediaData, error:e});
		
	}


	//############################################//
	/* google analytics */
	//############################################//

	if(useGa){

        if(!window["ga"]){
			(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
			(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
			m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
			})(window,document,'script','https://www.google-analytics.com/analytics.js','ga');
	    }
        ga('create', settings.gaTrackingId, 'auto');
        ga('send', 'pageview');
	    

        //download
        wrapper.on('click','.hap-download',function(e){
	    	if(!componentInited) return false;

			if(window["ga"]){
				var title = $(this).closest('.hap-playlist-item').find('.hap-playlist-title').html();
				ga('send', {
			        hitType: 'event',
			        eventCategory: 'Modern audio player: ' + instanceName,
			        eventAction: 'downloaded',
			        eventLabel: 'title: ' + title,
			        nonInteraction: true
			    });
			}

	    });

    }


	//############################################//
	/* continous playback */
	//############################################//

	if(useContinousPlayback){
		//save data on page close
		var eventName = isIOS ? "pagehide" : "beforeunload",
		addListener = window.attachEvent || window.addEventListener;

		addListener(eventName, function (event) { 
		    if(window.event)window.event.cancelBubble = true;

	   		if(!componentInited) return false;
			if(!mediaType) return false;

     		var d = {
			    activePlaylist: settings.activePlaylist,
			    volume: volume,
                activeItem: _playlistManager.getCounter(),
			    resumeTime: self.getCurrentTime(),
			    autoPlay : mediaPlaying
			};
			//remember last song time for each song
			if(continousPlaybackTrackAllSongs){
				var i, arr = [], item;
				for(i = 0;i<playlistLength; i++){
					item = playlistDataArr[i].data; 
					if(item.start)arr.push({title: item.title, artist: item.artist, start: item.start});
				}
				d.lastPositionArr = arr;
			}
			localStorage.setItem(continousKey, JSON.stringify(d));

		});
	}else{
		localStorage.removeItem(continousKey);
	}

	//############################################//
	/* media session */
	//############################################//

	if(useMediaSession){

	    navigator.mediaSession.setActionHandler('play', function() {
	        self.playMedia();
	    });

	    navigator.mediaSession.setActionHandler('pause', function() {
	        self.pauseMedia();
	    });

	    navigator.mediaSession.setActionHandler('previoustrack', function() {
	        self.previousMedia();
	    });

	    navigator.mediaSession.setActionHandler('nexttrack', function() {
	      	self.nextMedia();
	    });

	    navigator.mediaSession.setActionHandler('seekbackward', function() {
	        self.seekBackward();
	    });

	    navigator.mediaSession.setActionHandler('seekforward', function() {
	        self.seekForward();
	    });

	}

	function updateMetadata(){

		navigator.mediaSession.metadata = new MediaMetadata({
	        title: currMediaData.title,
	        artist: currMediaData.artist || '',
	        album: currMediaData.album || '',
	        artwork: [{src:currMediaData.thumb || ''}]
	    });

	}

	//############################################//
	/* tooltip */
	//############################################//

	var tooltip = wrapper.find('.hap-tooltip');

	if(!isMobile){

		if(tooltip.length && wrapper.css('position') == 'static'){
			console.log("player css position is static, therefore tooltip might not work correctly. Please set wrapper css position to other than static if you use tooltip in player.");
		}

		wrapper.on("mouseenter", "[data-tooltip]", function(e){

			var item = $(this),
			main = wrapper[0].getBoundingClientRect(),
			element = item[0].getBoundingClientRect();

			tooltip.text(item.attr('data-tooltip'));

			var t = parseInt(element.top - main.top - tooltip.outerHeight()),
			l = parseInt(element.left - main.left - tooltip.outerWidth()/2 + item.outerWidth()/2);

			//keep within component

			if(l + tooltip.outerWidth() > wrapper.width())l = wrapper.width() - tooltip.outerWidth();
			else if(l < 0) l = 0;

			if(t + main.top < 0)t = parseInt(element.top - main.top + tooltip.outerHeight() + 15);//position below
			//else if(t < 0) t = 0;
			
			tooltip.css({
				left: l+'px',
				top: t+'px'
			}).show();

		}).on("mouseleave", "[data-tooltip]", function(e){
			tooltip.hide();
		});	

	}

	//addon for mobile
	function showTooltip(item){

		var main = wrapper[0].getBoundingClientRect(),
			element = item[0].getBoundingClientRect();

			tooltip.text(item.attr('data-tooltip'));

			var t = parseInt(element.top - main.top - tooltip.outerHeight()),
			l = parseInt(element.left - main.left - tooltip.outerWidth()/2 + item.outerWidth()/2);

			//keep within component

			if(l + tooltip.outerWidth() > wrapper.width())l = wrapper.width() - tooltip.outerWidth();
			else if(l < 0) l = 0;

			if(t + main.top < 0)t = parseInt(element.top - main.top + tooltip.outerHeight() + 15);//position below
			//else if(t < 0) t = 0;
			
			tooltip.css({
				left: l+'px',
				top: t+'px'
			}).show();

	}


	//############################################//
	/* circle player */
	//############################################//
	
	var circleSeekbar = wrapper.find('.hap-circle-player').length,
	_HAPCirclePlayer;

	if(circleSeekbar && HAPUtils.hasCanvas()){
		if(typeof HAPCirclePlayer === 'undefined'){
			alert('Link to circleplayer.js file missing in head tag!');
		}else{
			_HAPCirclePlayer = new HAPCirclePlayer({settings:settings, parent:wrapper});
		}
		
	}


	//############################################//
	/* seekbar */
	//############################################//

	if(!circleSeekbar){
		var seekbar = wrapper.find('.hap-seekbar');
	}else{
		var seekbar = wrapper.find('.hap-progress-canvas');
	}

	seekbar.on(events.downEvent,function(e){
		if(settings.disableSeekbarInRange && rangeOn)return false;
		if(seekbarDisabled) return false;
		if(disableSongSkip) return false;
		if(_HAPAdManager && _HAPAdManager.isAdOn()) return false;
		if(!mediaType) return false;
		onDragStartSeek(e);
		return false;		
	});

	// Start dragging 
	function onDragStartSeek(e) {
		if(!componentInited) return;

		if(!seekBarDown){	
			var point;
			if(e.type == 'touchstart'){
				var currTouches = e.originalEvent.touches;
				if(currTouches && currTouches.length > 0) {
					point = currTouches[0];
				}else{	
					return false;						
				}
			}else{
				point = e;								
				e.preventDefault();						
			}
			seekBarDown = true;
			_doc.on(events.moveEvent, function(e) { onDragMoveSeek(e); }).on(events.upEvent, function(e) { onDragReleaseSeek(e); });		
		}
		return false;	
	}
				
	function onDragMoveSeek(e) {	
		var point;
		if(e.type == 'touchmove'){
			var touches;
			if(e.originalEvent.touches && e.originalEvent.touches.length) {
				touches = e.originalEvent.touches;
			}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
				touches = e.originalEvent.changedTouches;
			}else{
				return false;
			}
			// If touches more then one, so stop sliding and allow browser do default action
			if(touches.length > 1) {
				return false;
			}
			point = touches[0];	
			e.preventDefault();				
		} else {
			point = e;
			e.preventDefault();		
		}
		setProgress(point);
		
		return false;		
	}
	
	function onDragReleaseSeek(e) {
		if(seekBarDown){	
			seekBarDown = false;			
			_doc.off(events.moveEvent).off(events.upEvent);	
			
			var point;
			if(e.type == 'touchend'){
				var touches;
				if(e.originalEvent.touches && e.originalEvent.touches.length) {
					touches = e.originalEvent.touches;
				}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
					touches = e.originalEvent.changedTouches;
				}else{
					return false;
				}
				// If touches more then one, so stop sliding and allow browser do default action
				if(touches.length > 1) {
					return false;
				}
				point = touches[0];	
				e.preventDefault();				
			} else {
				point = e;
				e.preventDefault();		
			}
			setProgress(point, true);

			tooltip.hide();
		}
		
		return false;
	}	

	function setProgress(point, update) {

		if(!circleSeekbar){

			var seekPercent = point.pageX - progressBg.offset().left;
			if(seekPercent < 0) seekPercent = 0;
			else if(seekPercent > seekBarSize) seekPercent = seekBarSize;
			var v = Math.max(0, Math.min(1, seekPercent / seekBarSize));

		}else{

			var v = _HAPCirclePlayer.setProgress(point);

		}

		if(update){

			var t,d;

			if(mediaType == 'youtube'){
				
				if(youtubePlayer && youtubeReady)youtubePlayer.seekTo(v * youtubePlayer.getDuration());

			}else if(mediaType == 'audio'){

				if(audioUp2Js && !isNaN(audioUp2Js.duration)){

					d = audioUp2Js.duration;
					t = v * d;
					if(t > d - 2)t = d - 2;//prevent seeking to end
					
					try{
						audioUp2Js.currentTime = t;
					}catch(er){console.log(er)}
				
				}
			}

			if(!mediaStarted)progressLevel.width(v * seekBarSize);

		}else{

			progressLevel.width(v * seekBarSize);//redraw seekbar while user seeking, we dont want to update seek position until seeking ends for performance
		}

	}

	//############################################//
	/* seekbar tooltip */
	//############################################//

	if(!isMobile){
		seekbar.on('mouseover', function mouseOverHandlerSeek(){
			if(seekBarDown)return;
			if(_HAPAdManager && _HAPAdManager.isAdOn()) return false;
			seekbar.on(events.moveEvent, mouseMoveHandlerSeekTooltip).on('mouseout', mouseOutHandlerSeek);
			_doc.on('mouseout', mouseOutHandlerSeek);
		});
		function mouseOutHandlerSeek() {
			seekbar.off(events.moveEvent, mouseMoveHandlerSeekTooltip).off('mouseout', mouseOutHandlerSeek);
			_doc.off('mouseout', mouseOutHandlerSeek);
			tooltip.hide();
		}
	}
	
	function mouseMoveHandlerSeekTooltip(e){

		if(!circleSeekbar){

			var s = e.pageX - progressBg.offset().left;
			if(!HAPUtils.isNumber(s))return false;
			if(s < 0) s = 0;
			else if(s > seekBarSize) s = seekBarSize;
			var newPercent = Math.max(0, Math.min(1, s / seekBarSize));
			
			var main = wrapper[0].getBoundingClientRect(),
				element = seekbar[0].getBoundingClientRect();

			var l = parseInt(e.pageX - _window.scrollLeft() - main.left - tooltip.outerWidth()/2),
				t = parseInt(element.top - main.top - tooltip.outerHeight(),10) - 10;

			//restrain
			if(l < element.left - main.left)l = element.left - main.left;
			else if(l > element.left - main.left + seekbar.outerWidth() - tooltip.outerWidth()) l = element.left - main.left + seekbar.outerWidth() - tooltip.outerWidth();
			
			if(t + main.top < 0)t = parseInt(element.top - main.top + tooltip.outerHeight() + 15);//position below
			//if(t < 0) t = 0;

			tooltip.css({
				left: l+'px',
				top: t+'px'
			});

		}else{

			var newPercent = _HAPCirclePlayer.trackTooltip(e);

			var main = wrapper[0].getBoundingClientRect(),
				element = seekbar[0].getBoundingClientRect();

			var t = parseInt(element.top - main.top - tooltip.outerHeight() - 20),
				l = parseInt(element.left - main.left - tooltip.outerWidth()/2 + seekbar.outerWidth()/2);

			//keep within component
			if(l + tooltip.outerWidth() > wrapper.width())l = wrapper.width() - tooltip.outerWidth();
			else if(l < 0) l = 0;

			if(t + main.top < 0)t = parseInt(element.top - main.top + tooltip.outerHeight() + 20);//position below
			//if(t < 0) t = 0;

			tooltip.css({
				left: l+'px',
				top: t+'px'
			});

		}

		if(!HAPUtils.isNumber(newPercent))return false;

		if(mediaType == 'youtube'){
			
			if(youtubePlayer && youtubeReady)var d = youtubePlayer.getDuration(), t = newPercent * d;

		}else if(mediaType == 'audio'){

			if(audioUp2Js){
				var d = audioUp2Js.duration, t = newPercent * d;
			}
		}

		if(!HAPUtils.isNumber(d) || !HAPUtils.isNumber(d)) return;

		tooltip.html(HAPUtils.formatTime(t)+' / '+HAPUtils.formatTime(d)).show();
		
	}
	
	//############################################//
	/* volume */
	//############################################//
		
	var lastVolume = 0.5,
	volumeBarDown,
	volumeVertical = volumeSeekbar.hasClass('hap-volume-vertical'),
	volumeSize = volumeVertical ? volumeBg.height() : volumeBg.width();

	if(volume<0)volume = 0;
	else if(volume>1)volume = 1;
	if(volume!=0)lastVolume = volume;

	if(!volumeCanBeSet){	
		//volumeSeekbar.remove();
		//volumeToggle.remove();
	}

	volumeSeekbar.on(events.downEvent,function(e){
		onDragStartVolume(e);
		return false;		
	});

	function onDragStartVolume(e) {
		if(!componentInited) return;
		if(!volumeBarDown){					
			var point;
			if(e.type == 'touchstart'){
				var currTouches = e.originalEvent.touches;
				if(currTouches && currTouches.length > 0) {
					point = currTouches[0];
				}else{	
					return false;						
				}
			}else{
				point = e;								
				e.preventDefault();						
			}
			volumeBarDown = true;
			_doc.on(events.moveEvent, function(e) { onDragMoveVolume(e); }).on(events.upEvent, function(e) { onDragReleaseVolume(e); });		
		}
		return false;	
	}
				
	function onDragMoveVolume(e) {	
		var point;
		if(e.type == 'touchmove'){
			var touches;
			if(e.originalEvent.touches && e.originalEvent.touches.length) {
				touches = e.originalEvent.touches;
			}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
				touches = e.originalEvent.changedTouches;
			}else{
				return false;
			}
			// If touches more then one, so stop sliding and allow browser do default action
			if(touches.length > 1) {
				return false;
			}
			point = touches[0];	
			e.preventDefault();				
		} else {
			point = e;
			e.preventDefault();		
		}
		volumeTo(point);
		
		return false;		
	}
	
	function onDragReleaseVolume(e) {
		if(volumeBarDown){	
			volumeBarDown = false;			
			_doc.off(events.moveEvent).off(events.upEvent);	
			
			var point;
			if(e.type == 'touchend'){
				var touches;
				if(e.originalEvent.touches && e.originalEvent.touches.length) {
					touches = e.originalEvent.touches;
				}else if(e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
					touches = e.originalEvent.changedTouches;
				}else{
					return false;
				}
				// If touches more then one, so stop sliding and allow browser do default action
				if(touches.length > 1) {
					return false;
				}
				point = touches[0];	
				e.preventDefault();				
			} else {
				point = e;
				e.preventDefault();		
			}
			volumeTo(point);

			tooltip.hide();
		}
		
		return false;
	}	

	function toggleMute(){
		if(!componentInited) return false;
		if(volume>0){
			lastVolume = volume;//remember last volume
			volume = 0;
		}else{
			volume = lastVolume;//restore last volume
		}
		setVolume();
	}

	function volumeTo(v) {
		if(volumeVertical){
			volume = Math.max(0, Math.min(1, (v.pageY - volumeBg.offset().top) / volumeSize));
			volume = 1 - volume;//reverse for up dir
		}else{
			volume = Math.max(0, Math.min(1, (v.pageX - volumeBg.offset().left) / volumeSize));
		} 
		setVolume();
	}

	function setVolume(v){

		if(typeof v !== 'undefined')volume = v;

		if(mediaType == 'youtube'){
			if(youtubePlayer && youtubeReady){
				youtubePlayer.setVolume(volume * 100);
			}
		}else{
			if(audioUp2Js){
				audioUp2Js.volume = volume;
				if(volume == 0)audioUp2Js.muted = true;
				else audioUp2Js.muted = false;
			}
		}

		var prop = volumeVertical ? 'height' : 'width';
		if(!HAPUtils.isNumber(volumeSize))volumeSize = volumeVertical ? volumeBg.height() : volumeBg.width();
		volumeLevel.css(prop, volume*volumeSize+'px');

		volumeToggle.children().hide();//hide all volume btns

		if(volume == 0){
			volumeToggle.find('.hap-btn-volume-off').show();
		}else if(volume > 0 && volume < 0.5){	
			volumeToggle.find('.hap-btn-volume-down').show();
		}else{
			volumeToggle.find('.hap-btn-volume-up').show();
		}

		$(self).trigger("volumeChange", {instance:self, instanceName:instanceName, volume:volume});

	}

	//############################################//
	/* volume tooltip */
	//############################################//

	if(!isMobile){
		volumeSeekbar.on('mouseover', function mouseOverHandlerSeek(){
			if(volumeBarDown)return;
			volumeSeekbar.on(events.moveEvent, mouseMoveHandlerVolumeTooltip).on('mouseout', mouseOutHandlerVolume);
			_doc.on('mouseout', mouseOutHandlerVolume);
		});
		function mouseOutHandlerVolume() {
			volumeSeekbar.off(events.moveEvent, mouseMoveHandlerVolumeTooltip).off('mouseout', mouseOutHandlerVolume);
			_doc.off('mouseout', mouseOutHandlerVolume);
			tooltip.hide();
		}
	}
		
	function mouseMoveHandlerVolumeTooltip(e){

		if(!volumeVertical)var s = e.pageX - volumeBg.offset().left;
		else var s = e.pageY - volumeBg.offset().top;
		if(s<0) s=0;
		else if(s>volumeSize) s=volumeSize;
		var newPercent = Math.max(0, Math.min(1, s / volumeSize));
		if(!HAPUtils.isNumber(newPercent))return false;
		if(volumeVertical)newPercent = 1 - newPercent;//reverse for up dir
		var value=parseInt(newPercent * 100, 10);

		tooltip.text(value+' %');

		var main = wrapper[0].getBoundingClientRect(),
		element = volumeSeekbar[0].getBoundingClientRect();

		if(volumeVertical){
			var l = parseInt(element.left - main.left - tooltip.outerWidth()/2 + volumeSeekbar.outerWidth()/2),
				t = parseInt(e.pageY - _window.scrollTop() - main.top - tooltip.outerHeight() - 20);

				//restrain
				if(t < element.top - main.top - tooltip.outerHeight() - 10)t = element.top - main.top - tooltip.outerHeight() - 10;
				else if(t > element.top - main.top + volumeSeekbar.outerHeight() - tooltip.outerHeight()) t = element.top - main.top + volumeSeekbar.outerHeight() - tooltip.outerHeight();

		}else{
			var l = parseInt(e.pageX - _window.scrollLeft() - main.left - tooltip.outerWidth()/2),
				t = parseInt(element.top - main.top - tooltip.outerHeight()) - 15;

				//restrain
				if(l < element.left - main.left)l = element.left - main.left;
				else if(l > element.left - main.left + volumeSeekbar.outerWidth() - tooltip.outerWidth()) l = element.left - main.left + volumeSeekbar.outerWidth() - tooltip.outerWidth();
		}

		if(l < 0 || t < 0){//if we have a seekbar that shows on hover it will hide when we hold and press outside seekbar.
			//tooltip.hide();
			//return;
		} 

		if(t + main.top < 0)t = parseInt(element.top - main.top + tooltip.outerHeight() + 20);//position below
		//if(t < 0) t = 0;
		
		tooltip.css({
			left: l+'px',
			top: t+'px'
		}).show();
	}

	//############################################//
	/* range (a-b loop) */
	//############################################//

	var range_handle_a = wrapper.find('.hap-range-handle-a'),
	range_handle_b = wrapper.find('.hap-range-handle-b'),
	range_handle_a_width = parseInt(range_handle_a.attr('data-width'),10),
	range_handle_b_width = parseInt(range_handle_b.attr('data-width'),10),
	range_a_time,
	range_b_time,
	range_a_pos,
	range_b_pos,
	rangeOn,
	useRange,
	rangeSet

	self.toggleRange = function(){//open / close range
		if(!mediaType)return;

		if(rangeOn){
			range_handle_a.css('display','none');
			range_handle_b.css('display','none');

			rangeSet = true;

		}else{
			range_handle_a.css('display','block');
			range_handle_b.css('display','block');
			
			range_a_pos = parseInt(range_handle_a.css('left'),10);
			range_b_pos = parseInt(range_handle_b.css('left'),10);
			range_a_time = getRangeTime(range_a_pos);
			range_b_time = getRangeTime(range_b_pos);

			rangeSet = false;
		}
		rangeOn = !rangeOn;

		if(rangeOn)rangeToggle.addClass(hoverClass); 
		else rangeToggle.removeClass(hoverClass); 

	}

	function getRangeTime(x){//get media time by seek percent
		
		var v = Math.max(0, Math.min(1, x / seekBarSize)), c, d;

		if(mediaType == 'youtube'){
			d = youtubePlayer.getDuration();
		}else if(mediaType == 'audio'){
			d = audioUp2Js.duration;
		}
		c = v * d;
		return c;
	}

	//range api

	self.resetRange = function(){//reset range to default position
		rangeOn = false;
		rangeSet = true;
		range_handle_a.css({display:'none', left:0+'px'});
		range_handle_b.css({display:'none', left:seekBarSize+'px'});
		rangeToggle.removeClass(hoverClass); 
	}

	self.setRange = function(a,b){//set range
		
		var d = self.getDuration();

		if(typeof d === 'undefined')return false;

		if(a < 0)a = 0;
		if(b > d)b = d;
		
		range_a_time = a;
		range_b_time = b;

		rangeSet = false;

		range_handle_a.css('left',((range_a_time / d) * seekBarSize)+'px');
		range_handle_b.css('left',((range_b_time / d) * seekBarSize)+'px');

		if(!rangeOn)self.toggleRange();

	}

	self.getRange = function(){//get range times
		if(typeof range_a_time !== 'undefined' && typeof range_b_time !== 'undefined')return [range_a_time, range_b_time];
		else return null;
	}

	if(range_handle_a.length){

		useRange = true;

		_HAPRangeSlider = new HAPRangeSlider({
			settings:settings,
			range_handle_a:range_handle_a, 
			range_handle_b:range_handle_b, 
		});

		$(_HAPRangeSlider).on('HAPRangeSlider.RANGE_CHANGE', function(e,data){

			var pos = data.point.pageX - progressBg.offset().left, m, elem = data.elem;
			if(pos < 0)pos = 0;
			else if(pos > seekBarSize)pos = seekBarSize;

			if(elem.hasClass('hap-range-handle-a')){

				if(pos > parseInt(range_handle_b.css('left'),10) - range_handle_b_width)pos = parseInt(range_handle_b.css('left'),10) - range_handle_a_width;
				range_a_pos = pos;
				range_a_time = getRangeTime(range_a_pos);

			}else{

				if(pos < parseInt(range_handle_a.css('left'),10) + range_handle_a_width)pos = parseInt(range_handle_a.css('left'),10) + range_handle_a_width;
				range_b_pos = pos;
				range_b_time = getRangeTime(range_b_pos);

			}

			elem.css('left',pos+'px');

			if(data.event)mouseMoveHandlerSeekTooltip(data.event);
			else tooltip.hide();

		});

		

	}



	//############################################//
	/* playback rate */
	//############################################//

	var playbackRateSeekbar = wrapper.find('.hap-playback-rate-seekbar')

	if(playbackRateSeekbar.length){

		_HAPPlaybackRateSlider = new HAPPlaybackRateSlider({
			settings:settings,
			wrapper: wrapper, 
			seekbar:playbackRateSeekbar, 
			sliderBg:wrapper.find('.hap-playback-rate-bg'), 
			sliderLevel:wrapper.find('.hap-playback-rate-level'), 
			isVertical:playbackRateSeekbar.hasClass('hap-vertical'), 
			tooltip: tooltip,
		});

		$(_HAPPlaybackRateSlider).on('HAPPlaybackRateSlider.RANGE_CHANGE', function(e,data){
			self.setPlaybackRate(data.value, true);
		});

		

	}


	//############################################//
	/* context menu */
	//############################################//

	if(settings.disableRightClickOverPlayer){

		wrapper.on("contextmenu", function(){
			return false;
		});

	}

	//############################################//
	/* controls */
	//############################################//

	if(useKeyboardNavigationForPlayback){

		_doc.off('keyup.hap').on('keyup.hap',function(e) {
			if(!componentInited) return false;
			
			var key = e.keyCode, target = $(e.target);
		
			if(key == 37) {//left arrow
			  	self.previousMedia();
			} 
			else if(key == 39) {//right arrow
				self.nextMedia();
			}
			else if(key == 32) {//space
				if(target.hasClass('hap-search-filter')) return false;
				self.togglePlayback();
				e.preventDefault();
			}
			else if(key == 77) {//m
				if(target.hasClass('hap-search-filter')) return false;
				toggleMute();
			}
		});	
	}
	
	var buttonArr=[
		wrapper.find('.hap-next-toggle'),
		wrapper.find('.hap-prev-toggle'),
		wrapper.find('.hap-playlist-toggle'),
		wrapper.find('.hap-playlist-close'),
		shareToggle,
		wrapper.find('.hap-share-close'),
		wrapper.find('.hap-share-item'),
		wrapper.find('.hap-playback-rate-close'),
		playbackRateToggle,
		rangeToggle,
		popupToggle,
		sortAlpha,
		playbackToggle,
		loopToggle,
		randomToggle,
		volumeToggle
	];

	var btn,len = buttonArr.length,i;
	for(i=0;i<len;i++){
		btn = $(buttonArr[i]).css('cursor', 'pointer').on('click', clickControls);
		//if(!isMobile)btn.on('mouseenter', overControls).on('mouseleave', outControls);
	}
	
	function clickPlaylistItem(e){
		if(!componentInited)return false;
		if(disableSongSkip) return false;
		if(_HAPAdManager && _HAPAdManager.isAdOn()) return false;

		var currentTarget = $(e.currentTarget);

		playlistClick = true;
		autoPlay = true;

		beforeSongAdvanceAction();

		if(isIOS && currMediaData.adMid && !iosAdMidFixStart)_HAPAdManager.forceAdMidAudio();//ads 

		//############################################//
		/* accordion */
		//############################################//

		if(accordionMode){
			//check if we are still in same accordion folder. If not, switch data to new playlist
			var aid = currentTarget.closest('.hap-accordion-item').attr('data-id');
			if(aid != accordioPlaylistId){

				accordioPlaylistId = aid;

				playlistDataArr = playlistDataAccordion[accordioPlaylistId].data;

				playlistLength = playlistDataArr.length;
				var i;
				for(i = 0;i < playlistLength; i++){
					playlistDataArr[i].id = i;
				}
			
				_playlistManager.setPlaylistItems(playlistLength);//we treat each accordion as new playlist

			}

		}

		//############################################//
		/* accordion */
		//############################################//

		var id = currentTarget.closest('.hap-playlist-item').attr('data-id');
		_playlistManager.processPlaylistRequest(id);

		if(class_list.indexOf('hap-art-narrow-light') > -1){
			self.togglePlaylist();	
		}

		$(self).trigger('clickPlaylistItem', {instance:self, instanceName:instanceName});
	}

	function overPlaylistItem(e){
		e.preventDefault();
		if(!componentInited) return false;

		$(e.currentTarget).closest('.hap-playlist-item').addClass('hap-playlist-item-selected');

		$(self).trigger('overPlaylistItem', {instance:self, instanceName:instanceName});
	}
	
	function outPlaylistItem(e){
		e.preventDefault();
		if(!componentInited) return false;

		var item = $(e.currentTarget).closest('.hap-playlist-item');

		if(!item.hasClass('hap-playlist-item-disabled'))item.removeClass('hap-playlist-item-selected');

		$(self).trigger('outPlaylistItem', {instance:self, instanceName:instanceName});
	}

	function clickControls(e){
		if(!componentInited) return false;
		
		var currentTarget = $(e.currentTarget);

		if(currentTarget.hasClass('hap-playlist-toggle') || currentTarget.hasClass('hap-playlist-close')){	

			self.togglePlaylist();

		}else if(currentTarget.hasClass('hap-prev-toggle')){	

			if(isIOS && currMediaData.adMid && !iosAdMidFixStart)_HAPAdManager.forceAdMidAudio();//ads 

			if(disableSongSkip) return false;
			if(_HAPAdManager && _HAPAdManager.isAdOn()) return false;

			self.previousMedia();

		}else if(currentTarget.hasClass('hap-playback-toggle')){	

			if(isIOS && currMediaData.adMid && !iosAdMidFixStart)_HAPAdManager.forceAdMidAudio();//ads 

			self.togglePlayback();

		}else if(currentTarget.hasClass('hap-next-toggle')){

			if(isIOS && currMediaData.adMid && !iosAdMidFixStart)_HAPAdManager.forceAdMidAudio();//ads 

			if(disableSongSkip) return false;
			if(_HAPAdManager && _HAPAdManager.isAdOn()) return false;

			self.nextMedia();

		}else if(currentTarget.hasClass('hap-range-toggle')){	

			self.toggleRange();

		}else if(currentTarget.hasClass('hap-playback-rate-toggle') || currentTarget.hasClass('hap-playback-rate-close')){	

			shareHolder.hide();
			playbackRateHolder.toggle();
			tooltip.hide();

			if(playbackRateHolder.is(':visible')){
				playbackRateToggle.addClass(hoverClass); 
			}else{
				playbackRateToggle.removeClass(hoverClass);
			}

			shareToggle.removeClass(hoverClass);

		}else if(currentTarget.hasClass('hap-loop-toggle')){	

			loopToggle.find('.hap-btn').hide();
			loopStateCounter++;
			if(loopStateCounter > loopStateArr.length - 1)loopStateCounter = 0;
			loopState = loopStateArr[loopStateCounter];
			loopToggle.find('.hap-btn-loop-'+loopState).show();

			_playlistManager.setLooping(loopState);

			showTooltip(loopToggle.find('.hap-btn:visible'));

		}else if(currentTarget.hasClass('hap-random-toggle')){	

			randomPlay=!randomPlay;
			_playlistManager.setRandom(randomPlay);

			randomToggle.find('.hap-btn').hide();
			if(randomPlay)randomToggle.find('.hap-btn-random-on').show();
			else randomToggle.find('.hap-btn-random-off').show();

			showTooltip(randomToggle.find('.hap-btn:visible'));

		}else if(currentTarget.hasClass('hap-sort-alpha')){	

			if(HAPUtils.isEmpty(sortOrder))sortOrder = 'title-asc';
			else if(sortOrder == 'title-asc')sortOrder = 'title-desc';
			else if(sortOrder == 'title-desc')sortOrder = 'title-asc';
			self.sort(sortOrder);

			showTooltip(sortAlpha.find('.hap-btn:visible'));

		}else if(currentTarget.hasClass('hap-popup-toggle')){	

			if(settings.isPopup) return false;
			hapOpenPopup(settings, self);

		}else if(currentTarget.hasClass('hap-volume-toggle')){	

			if(!isMobile || volumeSeekbar.length == 0){
				toggleMute();
			}

		}else if(currentTarget.hasClass('hap-share-toggle') || currentTarget.hasClass('hap-share-close')){	

			shareHolder.toggle();
			playbackRateHolder.hide();
			tooltip.hide();

			playbackRateToggle.removeClass(hoverClass);

			if(shareHolder.is(':visible')){
				shareToggle.addClass(hoverClass); 
			}else{
				shareToggle.removeClass(hoverClass);
			}

		}else if(currentTarget.hasClass('hap-share-item')){		
			if(!mediaType)return false;
			_HAPShareManager.share(currentTarget.attr('data-type').toLowerCase(), currMediaData);
		}
	}
	
	function enableActivePlaylistItem(){

		if(activePlaylistItem){
			$(self).trigger('playlistItemEnabled', {instance:self, instanceName:instanceName, item:activePlaylistItem});

			activePlaylistItem.removeClass('hap-playlist-item-selected hap-playlist-item-disabled');
			activePlaylistItem = null;
		}
	}

	function disableActivePlaylistItem(id){

		if(activePlaylistItem){//enable active playlist item
			enableActivePlaylistItem();
		}

		activePlaylistItem = playlistContent.children('.hap-playlist-item').eq(id);

		if(activePlaylistItem.length){

			activePlaylistItem.addClass('hap-playlist-item-selected hap-playlist-item-disabled');

			if(usePlaylistScroll && playlistLength > 0){//scroll to active item

				if(!playlistClick){//not on playlist item click

					if(typeof mCustomScrollbar !== 'undefined'){
						if(playlistScrollOrientation == 'horizontal'){
							playlistInner.mCustomScrollbar("scrollTo",parseInt(activePlaylistItem.position().left),{scrollInertia:500});
						}else{
							playlistInner.mCustomScrollbar("scrollTo",parseInt(activePlaylistItem.position().top),{scrollInertia:500});
						}
					}else{
						var interval = setInterval(function(){//on start if we with for mCustomScrollbar to load
							if(typeof mCustomScrollbar !== 'undefined'){
								clearInterval(interval);

								if(playlistScrollOrientation == 'horizontal'){
									playlistInner.mCustomScrollbar("scrollTo",parseInt(activePlaylistItem.position().left),{scrollInertia:500});
								}else{
									playlistInner.mCustomScrollbar("scrollTo",parseInt(activePlaylistItem.position().top),{scrollInertia:500});
								}

							}
						},500);	
					}
				}
				playlistClick = false;
				
			}

			if(class_list.indexOf('hap-wall2') > -1){

				//append circle player and play btn to active playlist item (we cant append to item content because click is attached to it)
                wrapper.find('.hap-circle-player').addClass('hap-circle-player-set').appendTo(activePlaylistItem);
                wrapper.find('.hap-playback-toggle-wrapper').show().appendTo(activePlaylistItem);

			}

			$(self).trigger('playlistItemDisabled', {instance:self, instanceName:instanceName, item:activePlaylistItem});
		}
	}

	//############################################//
	/* playlist process */
	//############################################//
	
	function setPlaylist(id){

		playlistTransitionOn = true;
		preloader.show();
		
		if(lastPlaylist)destroyPlaylist();

		if(playlistList.length){
			var playlist = playlistList.find(id);
		}else{
			var playlist = $(id);
		}
		if(playlist.length == 0){
			alert('Failed playlist selection! Playlist - ' + id + ' does not exist. Check activePlaylist option in settings!');
			playlistTransitionOn = false;
			preloader.hide();
			return false;
		}

		settings.activePlaylist = id;

		$(self).trigger('playlistStartLoad', {instance:self, instanceName:instanceName});

		//global playlist options
		if(playlist.find('.hap-playlist-options').length){
			playlist.find('.hap-playlist-options').clone().prependTo(playlistContent);

			if(playlist.find('.hap-playlist-options').attr('data-media-prefix-url') != undefined){//for thumbs in build playlist
				mediaPrefixUrl = playlist.find('.hap-playlist-options').attr('data-media-prefix-url');
			}
		}

		playlistId = null;
		if(playlist.attr('data-playlist-id') != undefined){
			playlistId = playlist.attr('data-playlist-id');
			settings.playlistId = playlistId;
		}

		playlist.children('.hap-playlist-item').each(function() { 
			playlistProcessDataUrl.push(getTrackData($(this)));
		});

		playlistLength = playlistProcessDataUrl.length;
		checkProcessCounter();

	}

	//############################################//
	/* xml */
	//############################################//
	
	function processXml(data){
		//first load xml file, then process its content 
		console.log(data)
		$.ajax({
			type: 'GET',
			url: data.path,
			dataType: "html"
		}).done(function(result) {

			var obj = {};

			$(result).children('.hap-playlist-item').each(function() { 
				obj = getTrackData($(this));

				if(data.origtype)obj.origtype = data.origtype;
				if(data.mediaId !== 'undefined')obj.mediaId = data.mediaId;//for stats

				playlistProcessDataUrl.push(obj);
			});

			playlistLength = playlistProcessDataUrl.length;
			checkProcessCounter();
		  
		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processXml: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	}

	//############################################//
	/* m3u */
	//############################################//

	function processM3u(data) {

		if(window.location.protocol == 'file:'){
			alert('Reading m3u files requires server connection.');
			return false;
		}

		//note, no mixed playlists!
		//only audio file type
		
		$.ajax({
            type: 'GET',
            url: data.path,
        }).done(function(result) {

	        var obj, channel;

	        result
	        .replace('#EXTM3U', '')
	        .split('#EXTINF:')
	        .slice(1)
	        .map(function(str, index) {

	        	obj = $.extend(true, {}, data);
	            
	            obj.type = 'audio';

	            channel = str.split('\n').slice(0,-1);

	            //audio url
	            obj.mp3 = $.trim(channel[1]);

	            //title, artist 
	            var t = channel[0].split(',')
	            if(t.length > 1){
	            	var d = t[0];
	            	if(HAPUtils.isNumber(d) && d != -1)obj.duration = d;
	            	var title = t[1];
	            }else{
	            	var title = t[0];
	            }

	            if(title.indexOf('-')>-1){
	                var s = title.split('-')
	                obj.title = $.trim(s[1]);

	                if(s[0].match(/^\d+(\s*)+\./)) {//contains numbers in front of artist
		                var a = s[0].substr(s[0].indexOf('.')+1);
		                obj.artist = $.trim(a[1]);
	                }else{
	                	obj.artist = $.trim(s[0]);
	                }
	            }else{
	                obj.title = $.trim(title);
	            }

	            playlistProcessData.push(obj);

	        });

	       checkProcessCounter();

        }).fail(function(jqXHR, textStatus, errorThrown) {
            console.log('Error process m3u: ' + jqXHR, textStatus, errorThrown);
            checkProcessCounter();
        }); 

    }

	//############################################//
	/* json */
	//############################################//
	
	function processJson(data){
		//first load json file, then process its content 
		
		$.ajax({
			type: 'GET',
			url: data.path,
			dataType: "json"
		}).done(function(track) {

			playlistProcessCounter = -1;
			playlistProcessData = [];
			playlistProcessDataUrl = [];

			playlistFirstInit = false;
			if(!lastPlaylist)playlistFirstInit = true;
			lastPlaylist = playlistContent;

			if(Array.isArray(track))playlistProcessDataUrl = track;
			else playlistProcessDataUrl.push(track);

			if(data.origtype && data.mediaId !== 'undefined'){
				var i, len = playlistProcessDataUrl.length;
				for(i = 0;i<len;i++){
					playlistProcessDataUrl[i].origtype = data.origtype;
					if(!playlistProcessDataUrl[i].mediaId !== 'undefined')playlistProcessDataUrl[i].mediaId = data.mediaId;//for stats
				}
			}

			playlistLength = playlistProcessDataUrl.length;
			
			checkProcessCounter();
	  
		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processJson: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	}

	//############################################//
	/*  */
	//############################################//

	function checkProcessCounter() {
		playlistProcessCounter++;

		if(playlistProcessCounter > playlistLength - 1){
			buildPlaylist();
		}else{
			var data = playlistProcessDataUrl[playlistProcessCounter], type = data.type;

			if(type == 'soundcloud') {

				loadMoreType = 'soundcloud';

				if(data.limit){
					resultLimit = data.limit;
					if(data.loadMore)loadMoreOnTotalScroll = true;//from addtrack
				}else{
					resultLimit = allResults;
					loadMoreOnTotalScroll = false;
				} 

				soundCloudResultArr = [];

				if(!window.SC){
					initSoundCloud();
					var interval = setInterval(function(){
						if(soundCloudInited){
							if(interval) clearInterval(interval);
							getSoundCloudPage(true, data.path);
						}
					}, 100);
				}else{
					getSoundCloudPage(true, data.path);
				}
		
			}else if(type == 'podcast') {

				processPodcast(data);

			}else if(type == 'itunes_podcast_music') {

				processiTunes(data);

			}else if(type == 'folder') {

				loadMoreType = 'folder';

				processFolder(data);

			}else if(type == 'folder_accordion') {

				accordionMode = true;

				processFolderAccordion(data);	

			}else if(type == 'gdrive_folder'){

				processGdriveFolder(data);

			}else if(type == 'hls') {

				playlistProcessData.push(data);
				checkProcessCounter();

			}else if(type == 'audio'){

				playlistProcessData.push(data);
				checkProcessCounter();

			}else if(type == 'shoutcast' || type == 'icecast' || type == 'radiojar'){

				playlistProcessData.push(data);
				checkProcessCounter();

			}else if(type == 'youtube_single' || type == 'youtube_playlist') {

				loadMoreType = 'youtube';

				if(data.loadMore)loadMoreOnTotalScroll = true;//from addtrack

				if(!_HAPYoutubeLoader)createLoader('youtube');
				_HAPYoutubeLoader.setData(data);

			}else if(type == 'xml'){

				processXml(data);

			}else if(type == 'json'){

				processJson(data);

			}else if(type == 'm3u'){

				processM3u(data);

			}
		}
	}

	//############################################//
	/* get track data */
	//############################################//

	function getTrackData(track){

		var obj = {};

		obj.type = obj.origtype = track.attr('data-type');
		obj.origclasses = track.attr('class');//copy classes

		if(!track.is(':empty'))obj.content = track.html();//copy any content to generated playlist item

		if(track.attr('data-media-id') != undefined){//stats
			obj.mediaId = parseInt(track.attr('data-media-id'),10);
		}

		if(track.attr('data-mp3') != undefined)obj.mp3 = track.attr('data-mp3');
		if(track.attr('data-wav') != undefined)obj.wav = track.attr('data-wav');
		if(track.attr('data-aac') != undefined)obj.aac = track.attr('data-aac');
		if(track.attr('data-path') != undefined){
			obj.path = track.attr('data-path');
			if(obj.path.indexOf(bsf_match) != -1)obj.path = atob(obj.path.substr(6));
		}

		//radio
		if(track.attr('data-mountpoint') != undefined){
			obj.mountpoint = track.attr('data-mountpoint');
		}
		if(track.attr('data-version') != undefined){
			obj.version = track.attr('data-version');
		}
		if(track.attr('data-sid') != undefined){
			obj.sid = track.attr('data-sid');
		}

		if(track.attr('data-limit') != undefined){
			obj.limit = Math.abs(parseInt(track.attr('data-limit'),10));
		}

		if(track.attr('data-thumb') != undefined){
			obj.thumb = track.attr('data-thumb');
		}
		if(track.attr('data-thumb-default') != undefined){
			obj.thumbDefault = track.attr('data-thumb-default');
		}
		if(track.attr('data-alt') != undefined){
			obj.alt = track.attr('data-alt');
		}
		if(track.attr('data-title') != undefined){
			obj.title = track.attr('data-title');
		}
		if(track.attr('data-description') != undefined){
			obj.description = track.attr('data-description');
		}
		if(track.attr('data-artist') != undefined){
			obj.artist = track.attr('data-artist');
		}
		if(track.attr('data-album') != undefined){
			obj.album = track.attr('data-album');
		}
		if(track.attr('data-download') != undefined){
			obj.download = track.attr('data-download');
		}
		if(track.attr('data-duration') != undefined){
			obj.duration = track.attr('data-duration');
		}
		if(track.attr('data-date') != undefined){
			obj.date = track.attr('data-date');
		}
		if(track.attr('data-id3') != undefined){
			obj.id3 = true;
		}
		if(track.attr('data-start') != undefined){
			obj.start = Math.abs(track.attr('data-start'));
		}
		if(track.attr('data-end') != undefined){
			obj.end = Math.abs(track.attr('data-end'));
		}
		obj.playbackRate = 1;
		if(track.attr('data-playback-rate') != undefined){
			if(!HAPUtils.isEmpty(track.attr('data-playback-rate'))){
				var pr = Number(track.attr('data-playback-rate'));
				obj.playbackRate = HAPUtils.valueLimit(pr, settings.playbackRateMin, settings.playbackRateMax);
			}
		}
		if(track.attr('data-link') != undefined){
			obj.link = track.attr('data-link');
			obj.target = '_blank';
			if(track.attr('data-target') != undefined){
				obj.target = track.attr('data-target');
			}
		}
		if(track.attr('data-share') != undefined){
			obj.share = track.attr('data-share');
		}	
		if(track.attr('data-sort') != undefined){
			obj.sort = track.attr('data-sort');
		}	
		if(track.attr('data-active-folder') != undefined){
			obj.activeFolder = track.attr('data-active-folder');
		}

		if(track.find('.hap-custom-playlist-item-content').length)obj.customContent = track.find('.hap-custom-playlist-item-content').html();//copy any content to generated playlist item

		if(track.attr('data-load-more') != undefined){
			loadMoreOnTotalScroll = true;
		}
		

		//ads
		if(track.attr('data-ad-pre') != undefined){
			obj.adPre = track.attr('data-ad-pre').split(",").map(function(item){
			    return item.trim();
			});
			if(track.attr('data-shuffle-ads') != undefined && obj.adPre.length>1)HAPUtils.shuffleArray(obj.adPre);
		}
		if(track.attr('data-ad-mid') != undefined){
			obj.adMid = track.attr('data-ad-mid').split(",").map(function(item){
			    return item.trim();
			});
			if(track.attr('data-shuffle-ads') != undefined && obj.adMid.length>1)HAPUtils.shuffleArray(obj.adMid);
			if(track.attr('data-ad-mid-interval') != undefined){
				obj.adMidInterval = track.attr('data-ad-mid-interval');
			}
		}
		if(track.attr('data-ad-end') != undefined){
			obj.adEnd = track.attr('data-ad-end').split(",").map(function(item){
			    return item.trim();
			});
			if(track.attr('data-shuffle-ads') != undefined && obj.adEnd.length>1)HAPUtils.shuffleArray(obj.adEnd);
		}
		//ads
		

		return obj;
	}

	//############################################//
	/* youtube */
	//############################################//

	function createLoader(type){

		if(type == 'youtube'){

			_HAPYoutubeLoader = new HAPYoutubeLoader(settings);
			$(_HAPYoutubeLoader).on('HAPYoutubeLoader.END_LOAD', function(e, data){

				var i, len = data.data.length, obj;
				for(i=0;i<len;i++){
					obj = data.data[i];
					playlistProcessData.push(obj);
				}

				nextPageToken = data.nextPageToken;

				if(loadMoreProcess){
					buildPlaylist();
				}else{
					checkProcessCounter();
				}
				
			});

		}
	}

	//############################################//
	/* folder */
	//############################################//

	function processFolder(data){

		if(window.location.protocol == 'file:'){
			alert('Reading files from folders requires server connection.');
			return false;
		}

		if(data.limit){
			resultLimit = data.limit;
			if(data.loadMore)loadMoreOnTotalScroll = true;//from addtrack
		}else{
			resultLimit = allResults;
		} 

		var path = data.path.replace(/\/\//g, "/"), url = sourcePath + 'includes/folder_parser.php', ajax_data = {dir: path}, limit = resultLimit, load_more_limit = resultLimit;

		nextPageToken = [];

		if(data.id3){
			id3Start = id3Counter = playlistProcessData.length-1;
			useId3 = true;
		}else{
			useId3 = false;
		}

		$.ajax({
			type: 'GET',
			url: url,
			data: ajax_data,
			dataType: "json"
		}).done(function(media) {

			var i, len = media.length, entry, obj, full_path;

			if(data.sort){
				if(data.sort == 'filename-asc')HAPUtils.keysrt(media, 'filename');
				else if(data.sort == 'filename-desc')HAPUtils.keysrt(media, 'filename', true);
				else if(data.sort == 'date-asc')HAPUtils.keysrt(media, 'filemtime');
				else if(data.sort == 'date-desc')HAPUtils.keysrt(media, 'filemtime', true);
			}

			for(i=0; i < len; i++){

				entry = media[i];

				obj= $.extend(true, {}, data);
				obj.type = 'audio';

				full_path = entry.fullpath;
				obj[entry.extension] = full_path;

				if(createDownloadIconsInPlaylist)if(!obj.download)obj.download = full_path;
				if(!obj.share)obj.share = full_path;
				if(createLinkIconsInPlaylist)if(!obj.link)obj.link = full_path;

				if(!obj.title)obj.title = entry.filename;

				if(i < load_more_limit){
					playlistProcessData.push(obj);
					id3Counter++; 
				}else{
					nextPageToken.push(obj);
				}
				
			}

			if(nextPageToken.length == 0){
				loadMoreOnTotalScroll = false;
				nextPageToken = null;
			}

			if(data.id3){
				if(typeof jsmediatags === 'undefined'){
        			alert('Link to jsmediatags.js missing in head tag!')
        		}
				getId3();
			}else{
				checkProcessCounter();
			} 
	  
		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processFolder: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	}

	//############################################//
	/* accordion */
	//############################################//

	function processFolderAccordion(data){

		if(window.location.protocol == 'file:'){
			alert('Reading files from folders requires server connection.');
			return false;
		}

		var path = data.path.replace(/\/\//g, "/"), url = sourcePath + 'includes/folder_accordion.php', ajax_data = {dir: path};

		$.ajax({
			type: 'GET',
			url: url,
			data: ajax_data,
			dataType: "json"
		}).done(function(media) {

			var i, len = media.length, len2, j, parent, entry, children, obj, full_path;

			for(i=0; i < len; i++){

				children = media[i].children;

				playlistDataAccordion.push({media:children})

				//sort each folder individually
				if(data.sort){
					if(data.sort == 'filename-asc')HAPUtils.keysrt(children, 'filename');
					else if(data.sort == 'filename-desc')HAPUtils.keysrt(children, 'filename', true);
					else if(data.sort == 'date-asc')HAPUtils.keysrt(children, 'filemtime');
					else if(data.sort == 'date-desc')HAPUtils.keysrt(children, 'filemtime', true);
				}

				len2 = children.length;

				for(j=0; j < len2; j++){

					entry = children[j];

					obj = {type: 'audio'};
					if(data.mediaId !== 'undefined')obj.mediaId = data.mediaId;

					full_path = entry.fullpath;
					obj[entry.extension] = full_path;

					if(createDownloadIconsInPlaylist)if(!obj.download)obj.download = full_path;
					if(!obj.share)obj.share = full_path;
					if(createLinkIconsInPlaylist)if(!obj.link)obj.link = full_path;

					if(!obj.title)obj.title = entry.filename;

					children[j] = obj;

				}
					
			}

			buildAccordion(media, data.activeFolder);//create accordion parents

			//set accordion item active on start
			if(!accordionActiveId)accordionActiveId = 0;
			accordioPlaylistId = accordionActiveId;

			accordionHolder = playlistContent.find('.hap-accordion-item[data-id="'+accordionActiveId+'"]').addClass('hap-accordion-item-inited hap-accordion-item-opened').find('.hap-accordion-item-content');

			playlistContent = accordionHolder;

			playlistProcessData = media[accordionActiveId].children;

			if(data.id3){
				accordionUseId3 = true;
				if(typeof jsmediatags === 'undefined'){
        			alert('Link to jsmediatags.js missing in head tag!')
        		}
        		id3Start = -1;
        		id3Counter = playlistProcessData.length-1;
				getId3_all();
			}else{
				buildPlaylist();
			}
	  
		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processFolderAccordion: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	}

	//############################################//
	/* google drive folder */
	//############################################//

	function processGdriveFolder(data){

		if(window.location.protocol == 'file:'){
			alert('Reading files from folders locally is not possible! This requires server connection.');
			return false;
		}

		if(HAPUtils.isEmpty(gDriveAppId)){
			alert('gDriveAppId has not been set in settings!');
			return false;
		}

		var url = "https://www.googleapis.com/drive/v3/files?q='" + data.path + "'+in+parents&pageSize=1000&key=" + gDriveAppId;

		$.ajax({
			url: url,
			dataType: "jsonp"
		}).done(function(media) {

			var i, len = media.files.length, entry, obj, imgArr = [], mediaArr = [];

			for(i=0; i < len; i++){

				entry = media.files[i];

				//split audio and images (assume there is equal number of audio and image files named the same in folder!)
				if(/mp3|mpeg|mpeg3|wav|aac|adts/.test(entry.mimeType)){	
					mediaArr.push(entry);
				}else if(/jpg|jpeg|png/.test(entry.mimeType)){	
					imgArr.push(entry);
				}
			}

			if(data.sort){
				if(data.sort == 'filename-asc'){
					HAPUtils.keysrt(mediaArr, 'name');
					HAPUtils.keysrt(imgArr, 'name');
				}
				else if(data.sort == 'filename-desc'){
					HAPUtils.keysrt(mediaArr, 'name', true);	
					HAPUtils.keysrt(imgArr, 'name', true);
				}
			}

			len = mediaArr.length;

			for(i=0; i < len; i++){

				entry = mediaArr[i];

				obj= $.extend(true, {}, data);
				obj.type = 'audio';

				var extension = entry.name.substr(entry.name.lastIndexOf('.')+1);//use mime type maybe?
				obj[extension.toLowerCase()] = 'https://drive.google.com/uc?export=view&id=' + entry.id;

				if(createDownloadIconsInPlaylist)if(!obj.download)obj.download = 'https://drive.google.com/uc?export=download&id=' + entry.id;
				if(!obj.share)obj.share = 'https://drive.google.com/open?id=' + entry.id;
				if(createLinkIconsInPlaylist)if(!obj.link)obj.link = 'https://drive.google.com/open?id=' + entry.id;

				if(!obj.thumb && imgArr[i])obj.thumb = 'https://drive.google.com/uc?export=view&id=' + imgArr[i].id;

				if(!obj.title)obj.title = entry.name.substr(0, entry.name.lastIndexOf('.'));

				playlistProcessData.push(obj); 

			}

			checkProcessCounter();
	  
		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processGdriveFolder: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	}

	function resumeFolderLoad(){
		if(!nextPageToken)return false;

		playlistTransitionOn = true;
		preloader.show();
		playlistProcessData = nextPageToken.splice(0,resultLimit);
		if(nextPageToken.length == 0)nextPageToken = null;

		//check id3
		if(useId3){
			id3Start = -1;
			id3Counter = playlistProcessData.length-1;
			getId3();
		}else{
			buildPlaylist();
		}
		
	}

	//############################################//
	/* id3 tags */
	//############################################//

	function getId3(){
		/*https://github.com/aadsm/jsmediatags*/

		var item = playlistProcessData[id3Counter],
		url = item.mp3 || item.wav || item.aac;

		jsmediatags.read(url, {
		    onSuccess: function(tag) {
		        var tags = tag.tags, image = tags.picture;

		        if(tags.artist) item.artist = tags.artist;
				if(tags.title) item.title = tags.title;
				if(tags.album) item.album = tags.album;
				if(image){
					var base64String = "", i, len = image.data.length;
					for(i = 0; i < len; i++)base64String += String.fromCharCode(image.data[i]);
					item.thumb = "data:" + image.format + ";base64," + window.btoa(base64String);
				}
				if(tags.TLEN){//https://community.mp3tag.de/t/calculate-mp3-length/10145
					item.duration = tags.TLEN.data / 1000;
				}

				id3Counter--;
				if(id3Counter > id3Start){
					getId3();  
				}else{
					if(loadMoreProcess){
						buildPlaylist();//from resume folder load
					}else{
						checkProcessCounter();
					}
					
				}

		    },
		    onError: function(error) {
		        console.log("ID3 error: ", error.type, error.info);
		        id3Counter--;
				if(id3Counter > id3Start){
					getId3();  
				}else{
					checkProcessCounter();
				}  
		    }
		});
	}

	//############################################//
	/* accordion */
	//############################################//

	function getId3_all(){

		var item = playlistProcessData[id3Counter],
		url = item.mp3 || item.wav || item.aac;

		jsmediatags.read(url, {
		    onSuccess: function(tag) {
		        var tags = tag.tags, image = tags.picture;

		        if(tags.artist) item.artist = tags.artist;
				if(tags.title) item.title = tags.title;
				if(tags.album) item.album = tags.album;
				if(image){
					var base64String = "", i, len = image.data.length;
					for(i = 0; i < len; i++)base64String += String.fromCharCode(image.data[i]);
					item.thumb = "data:" + image.format + ";base64," + window.btoa(base64String);
				}
				if(tags.TLEN){
					item.duration = tags.TLEN.data / 1000;
				}

				id3Counter--;
				if(id3Counter > id3Start){
					getId3_all();  
				}else{
					buildPlaylist();
				}

		    },
		    onError: function(error) {
		        console.log("ID3 error: ", error.type, error.info);
		        id3Counter--;
				if(id3Counter > id3Start){
					getId3_all();  
				}else{
					buildPlaylist();
				}  
		    }
		});
	}

	//############################################//
	/* podcast */
	//############################################//

	function processPodcast(data){

		if(window.location.protocol == 'file:'){
			alert('Using Podcast locally is not possible! This requires online connection!');
			return;
		}

		if(data.limit) resultLimit = data.limit;
		else resultLimit = allResults;

		var url = proxy + '?url='+ encodeURIComponent(data.path), limit = resultLimit;

		$.ajax({
			url: url,
			dataType: "json",
			cache: false 
		}).done(function( d ) {

			var dom = HAPUtils.parseXML(d.contents), item, obj, default_artwork, len = 0;

			if($(dom).find('image').length && $(dom).find('image').attr('href')){
				default_artwork = $(dom).find('image').attr('href');
			}else if($(d.contents).find('itunes\\:image').length && $(d.contents).find('itunes\\:image').attr('href')){
				default_artwork = $(d.contents).find('itunes\\:image').attr('href');
			}

			if(default_artwork && default_artwork.lastIndexOf('&size=Large'))default_artwork = default_artwork.replace("=Large", "=Small");

			$(dom).find("item").each(function(){

				if(len >= limit) return false;

				item = $(this);

				obj = $.extend(true, {}, data);
				obj.type = 'audio';

				obj.mp3 = item.find('enclosure').attr('url');

				if(!obj.title && item.find('title').length)obj.title = item.find('title').text();
				if(!obj.artist){
					if(item.find('author').length)obj.artist = item.find('author').text();
					else if(item.find('itunes\\:author').length) obj.artist = item.find('itunes\\:author').text();
				}  
				if(item.find('pubDate').length)obj.date = item.find('pubDate').text();
					
				if(!obj.description && item.find('description').length){
					var desc = item.find('description').text();
					var div = document.createElement("div");
					div.innerHTML = desc;
					obj.description = div.textContent || div.innerText || "";
				}
				if(!obj.duration){
					if(item.find('itunes\\:duration').length)obj.duration = item.find('itunes\\:duration').text();
					else if(item.find('duration').length)obj.duration = item.find('duration').text();
				}
				if(!obj.thumb){
					if(item.find('image').length && item.find('image').attr('href')){
						obj.thumb = item.find('image').attr('href');
					}
					else if(item.find('itunes\\:image').length && item.find('itunes\\:image').attr('href')){
						obj.thumb = item.find('itunes\\:image').attr('href');
					}
					else if(default_artwork){
						obj.thumb = default_artwork;
					}
				}

				if(obj.thumb.lastIndexOf('&size=Large'))obj.thumb = obj.thumb.replace("=Large", "=Small");

				if(createDownloadIconsInPlaylist)if(!obj.download)obj.download = obj.mp3;

				if(item.find('link').length){
					if(!obj.share)obj.share = item.find('link').text();
					if(!obj.link)if(createLinkIconsInPlaylist)obj.link = item.find('link').text();
				}

				playlistProcessData.push(obj); 

				len++;
			});

			checkProcessCounter();
			
		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processPodcast: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	
	}

	//############################################//
	/* itunes */
	//############################################//

	function processiTunes(data){

		if(window.location.protocol == 'file:'){
			alert('Using Podcast locally is not possible! This requires online connection!');
			return;
		}

		var match = data.path.match(/id(\d+)/); 
		if (match) var podID = match[1];
		else var podID = url.match(/\d+/); 
 
		if(!podID){
		  	console.log('No Podcast ID found!');
		  	checkProcessCounter();
		  	return;
		}

		$.ajax({
			url: 'https://itunes.apple.com/lookup',
			data: {
		        id: parseInt(podID),
		        entity: 'podcast'
		    },
		    type: 'GET',
		    dataType: 'jsonp',
		}).done(function(result) {

		    data.path = result.results[0]['feedUrl'];
		    processPodcast(data);

		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log('Error processiTunes: ' + jqXHR, textStatus, errorThrown);
			checkProcessCounter();
		});	
	
	}

	//############################################//
	/* soundcloud */
	//############################################//

	function initSoundCloud(){

		if(window.location.protocol == 'file:'){
			alert('Using SoundCloud locally is not possible! This requires server connection!');
			return;
		}

		if(HAPUtils.isEmpty(soundCloudAppId)){
			alert('soundCloudAppId has not been set in settings!');
			return false;	
		}

		var tag = document.createElement('script');
		tag.src = "https://connect.soundcloud.com/sdk.js";
		var firstScriptTag = document.getElementsByTagName('script')[0];
		firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
		
		var interval = setInterval(function(){
			if(window.SC){
				if(interval) clearInterval(interval);
				SC.initialize({
					client_id: soundCloudAppId
				});
				soundCloudInited = true;
			}
		}, 100);
	}

	function getSoundCloudPage(resolve, path){

		if(resolve)var id = 'https://api.soundcloud.com/resolve.json?url='+path+'&client_id='+soundCloudAppId;
		else var id = path;

		SC.get(id, { limit: resultLimit, linked_partitioning: 1 }, function(data, error){

			if(error){
				console.log('Error getSoundCloudPage: ' + error);
				while(soundCloudResultArr.length){
					playlistProcessData.push(soundCloudResultArr.splice(0,1)[0])
				}

				checkProcessCounter();
				return;
			}

			if(data.kind == "track") {

				getSoundCloudTrack(data);

			}else if(data.kind == "playlist"){
				//no pagination, returns some tracks but not all?

				var i, len = data.tracks.length, end;
				if(soundCloudResultArr.length + len >= resultLimit){
					len = resultLimit - soundCloudResultArr.length;
					end = true;
				}
			
				for(i=0; i<len; i++) {
					getSoundCloudTrack(data.tracks[i]);
				}

			}else if(data.collection){
				//has pagination
				var i, len = data.collection.length, end;
				if(soundCloudResultArr.length + len >= resultLimit){
					len = resultLimit - soundCloudResultArr.length;
					end = true;
				}
			
				for(i=0; i<len; i++) {
					getSoundCloudTrack(data.collection[i]);
				}

			}else if(data.username) {
				//data username goes to data collection, always?
				var sc_uri = data.uri;
				if(/likes/.test(path)) {
					sc_uri += '/likes';
				}else if(/favorites/.test(path)) {
					sc_uri += '/favorites';
				}else{
					sc_uri += '/tracks';
				}
				getSoundCloudPage(false, sc_uri);
				return;

			}else if($.isArray(data)) {
				//username/tracks returns this, always returns 50 tracks?, no pagination
				var i, len = data.length;
				if(soundCloudResultArr.length + len >= resultLimit){
					len = resultLimit - soundCloudResultArr.length;
					end = true;
				}
				for(i=0; i<len; i++) {
					getSoundCloudTrack(data[i]);
				}
			}

			if(data.next_href)nextPageToken = data.next_href;
			else nextPageToken = null;

			if(data.next_href && !end)getSoundCloudPage(false, data.next_href);
			else{
				while(soundCloudResultArr.length){
					playlistProcessData.push(soundCloudResultArr.splice(0,1)[0])
				}
				checkProcessCounter();
			} 
			
		});
	}

	function resumeSoundCloud(){
		if(!nextPageToken)return false;

		playlistTransitionOn = true;
		preloader.show();
		playlistProcessCounter = 0;
		playlistProcessDataUrl = [];
		playlistLength = 1;

		if(!window.SC){
			initSoundCloud();
			var interval = setInterval(function(){
				if(soundCloudInited){
					if(interval) clearInterval(interval);
					getSoundCloudPage(false, nextPageToken);
				}
			}, 100);
		}else{
			getSoundCloudPage(false, nextPageToken);
		}
	}
	
	function getSoundCloudTrack(track){

		if(track.streamable && track.stream_url){//http://stackoverflow.com/questions/15211538/tracks-for-the-hives-are-not-streaming-via-the-api

			var obj = $.extend(true, {}, playlistProcessDataUrl[playlistProcessCounter]);

			obj.type = 'audio';
			if(track.duration)obj.duration = track.duration / 1000;//miliseconds
			obj.mp3 = track.stream_url + '?client_id=' + soundCloudAppId;
			if(createDownloadIconsInPlaylist)if(!obj.download)if(track.downloadable && track.download_url)obj.download = obj.mp3.replace(/\/stream\\?/, "/download");
			if(!obj.title && track.title)obj.title = track.title;
			if(!obj.description && track.description)obj.description = track.description;
			if(!obj.thumb && track.artwork_url){
				obj.thumb = track.artwork_url;
			}
			if(track.created_at)obj.date = track.created_at;
			obj.favoritingsCount = parseInt(track.favoritings_count?track.favoritings_count:0,10);
			obj.playbackCount = parseInt(track.playback_count?track.playback_count:0,10);
			obj.hotness = obj.favoritingsCount + obj.playbackCount;
			if(track.permalink_url){
				if(!obj.share)obj.share = track.permalink_url;
				if(createLinkIconsInPlaylist)if(!obj.link)obj.link = track.permalink_url;
			}

			soundCloudResultArr.push(obj); 

		}
	}


	

	//############################################//
	/* build playlist */
	//############################################//

	function buildPlaylist() {

		var counter, i, j = !addTrackProcess ? playlistDataArr.length : insertPosition, len = playlistProcessData.length, type, div, item_content, data, thumbsrc, thumbImg, playlistThumb, title, alt, desc, date, duration, counter_add = 0, currentInsert, playlist_data = [];

		for (i=0; i < len; i++){

			counter = i+j;			
			if(addTrackProcess)counter_add++;

			data = playlistProcessData[i];
			//console.log(data)
			type = data.type;

			if(usePlaylist){	

				if(!data.origclasses)data.origclasses = 'hap-playlist-item';//copy classes if exist
				div = $('<div class="'+data.origclasses+'"/>').attr('data-type', type);
				delete data.origclasses;

				if(!addTrackProcess){
					div.appendTo(playlistContent);
				}else{
					if(!currentInsert){
						if(endInsert){
							div.appendTo(playlistContent);
						}else{
							playlistContent.children('div').eq(insertPosition).before(div);
						}
					}else{
						currentInsert.after(div);
					}
					currentInsert = div;
				}

				item_content = $('<div class="hap-playlist-item-content"/>').appendTo(div);

				//thumb
				if(playlistItemContent.indexOf('thumb') != -1){
					var thumb = data.thumb || data.thumbDefault;
					if(thumb){

						if(mediaPrefixUrl && !HAPUtils.relativePath(thumb))thumb = mediaPrefixUrl + thumb;

						var img = new Image();
						img.className = "hap-thumbimg";
						img.src = thumb;

						if(data.alt)alt = data.alt;
						else if(data.title)alt = data.title.replace(/"/g, "'");
						else alt = 'image';
						img.alt = alt;

						$('<span class="hap-playlist-thumb"></span>').append(img).appendTo(item_content);

					}
				}

				//title, description
				if(playlistItemContent.indexOf('title') != -1 && data.title || playlistItemContent.indexOf('description') != -1 && data.description){

					var info = $('<span class="hap-playlist-info">').appendTo(item_content);
					
					if(playlistItemContent.indexOf('title') != -1 && data.title){
						title = getTitle(data);

						var tmc = settings.multilineTitle ? ' hap-title-multiline' : '';
						if(useNumbersInPlaylist){
							$('<span class="hap-playlist-title-wrap"><span class="hap-playlist-title'+tmc+'">'+title+'</span></span>').appendTo(info);
						}else{
							$('<span class="hap-playlist-title'+tmc+'">'+title+'</span>').appendTo(info);
						}
					}
					if(playlistItemContent.indexOf('description') != -1 && data.description){
						desc = data.description;
						if(settings.limitDescriptionText){
							desc = desc.substr(0,settings.limitDescriptionText)+'...';
						}
						$('<span class="hap-playlist-description">'+desc+'</span>').appendTo(info);
					}
				}

				//duration, date
				if(playlistItemContent.indexOf('duration') != -1 && data.duration || playlistItemContent.indexOf('date') != -1 && data.date){

					var info2 = $('<span class="hap-playlist-info2">').appendTo(item_content);
					
					if(playlistItemContent.indexOf('duration') != -1 && data.duration){
						if(isNaN(data.duration)){
							if(data.duration.lastIndexOf(':')==-1)duration = HAPUtils.formatTime(data.duration);
							else duration = data.duration;
						}else{
							duration = HAPUtils.formatTime(data.duration);
						}
						$('<span class="hap-playlist-duration">'+duration+'</span>').appendTo(info2);
					}
					
					if(playlistItemContent.indexOf('date') != -1 && data.date){
						date = new Date(data.date).toDateString().slice(4,10);
						$('<span class="hap-playlist-date">'+date+'</span>').appendTo(info2);
					}
				}

				//we need data-media-id, title, and data-artist attributes for stats and api!
				if(data.mediaId !== 'undefined'){
					div.attr('data-media-id', data.mediaId);
				}
				if(data.title){
					div.attr('title', data.title);
					var ss = data.title.replace(/['"\|]/g,'');
					div.attr('data-safe-title', ss);
				}
				if(data.artist){
					div.attr('data-artist', data.artist);
					var ss = data.artist.replace(/['"\|]/g,'');
					div.attr('data-safe-artist', ss);
				}
				if(data.description){
					div.attr('data-description', data.description);
				}

				if(useStatistics && statisticsContent.length){
					
					//statistics icons
					var stats = $('<div class="hap-stats"></div>');
					item_content.after(stats);

					if(statisticsContent.indexOf('plays') != -1){
						var p = $('<div class="hap-play-count" data-tooltip="'+settings.tooltipStatPlays+'">'+statPlayIcon+'<span>0</span></div>');
						stats.append(p);
					}

					if(statisticsContent.indexOf('likes') != -1){
					    var p = $('<div class="hap-like-count" data-tooltip="'+settings.tooltipStatLikes+'">'+statLikeIcon+'<span>0</span></div>');
					    stats.append(p);
					}

					if(statisticsContent.indexOf('downloads') != -1){
					    if(hasDownloadSupport && data.download != undefined){

					    	var p = $('<a href="'+data.download+'" class="hap-download-count" download aria-label="'+settings.tooltipStatDownloads+'" data-tooltip="'+settings.tooltipStatDownloads+'">'+statDownloadIcon+'<span>0</span></a>');

						}else{
							//if download not set on item still show download icon count?
							var p = $('<div class="hap-download-count hap-no-download" data-tooltip="'+settings.tooltipStatDownloads+'">'+statDownloadIcon+'<span>0</span></div>');
						}
						stats.append(p);
					}

				}else{

					//download icon
					if(hasDownloadSupport && data.download){
						var d = $('<a class="hap-download" href="'+data.download+'" download title="'+downloadIconTitle+'" aria-label="'+settings.downloadIconTitle+'">'+downloadIcon+'</a>');
						item_content.after(d);
					}

				}

				//link icon
				if(data.link){
					var target = data.target || '_blank'; 
					p = $('<a class="hap-link" href="'+data.link+'" target="'+target+'" aria-label="link">'+linkIcon+'</a>');
					item_content.after(p);
				}

				if(data.customContent){//copy any content to generated playlist item
					div.append(data.customContent);
					delete data.customContent;
				}
			
				item_content.on('click', clickPlaylistItem);
				if(!isMobile)item_content.on('mouseenter', overPlaylistItem).on('mouseleave', outPlaylistItem);
			}

			if(accordionMode)playlist_data.push({id: i, type: type, data: data});
			else playlistDataArr.splice(counter, 0, {id: counter, type: type, data: data});

				
		}

		if(accordionMode){

			//############################################//
			/* accordion */
			//############################################//

			playlistDataAccordion[accordionActiveId].data = playlist_data;//save all accordion data (when we click other acordion item, we change data)

			//only first time accordion is created, next time we switch playlist only when song is clicked in other accordion

			if(!accordionInited){
				playlistDataArr = playlist_data;//current accordion playlist data

				updatePlaylist(true, true);	
			
				_playlistManager.setPlaylistItems(playlistLength);//we treat each accordion as new playlist

			}else{
				updatePlaylist(false, true);	
			}

			endInit();

			//open accordion item
			var content = accordionHolder[0];
       	    content.style.height = content.scrollHeight + "px";

			accordionLoading = false;//set accordion item loaded

			accordionInited = true;

			//############################################//
			/* accordion */
			//############################################//

		}else{
		
			lastPlaylist = playlistContent;

			updatePlaylist(true, true);	

			console.log('playlistDataArr = ', playlistDataArr);


			if(!addTrackProcess){
				_playlistManager.setPlaylistItems(playlistLength);
			}else{
				var current_counter = _playlistManager.getCounter();
				_playlistManager.setPlaylistItems(playlistLength, false);
				
				if(insertPosition <= current_counter){
					if(!endInsert)_playlistManager.reSetCounter(current_counter+counter_add);
				}
				if(playlistFirstInit){
					playlistFirstInit = false;	
					if(addTrackPlayit && !isMobile)autoPlay = true;
					_playlistManager.setCounter(insertPosition, false);
				}else if(addTrackPlayit){
					_playlistManager.setCounter(insertPosition, false);
				}
			}

			endInit();

		}
			
	}

	//############################################//
	/* accordion */
	//############################################//

	function checkAccordion(holder){//check accordion item when its clicked first time 

		preloader.show();

		accordionActiveId = holder.attr('data-id');
		accordionHolder = holder.addClass('hap-accordion-item-inited').find('.hap-accordion-item-content');
		playlistContent = accordionHolder;//current accordion playlist
		playlistProcessData = playlistDataAccordion[accordionActiveId].media;

		if(accordionUseId3){//load id3 tags
    		id3Start = -1;
    		id3Counter = playlistProcessData.length-1;
			getId3_all();
		}else{//we dont use id3 tags
			buildPlaylist();
		}
	}

	function buildAccordion(arr, activeFolder){//make accordion parents

		var i, len = arr.length, str = '', title;

		for(i=0; i < len; i++){
			title = arr[i].parent.replace(/"/g, "'");

			if(activeFolder !== 'undefined'){//active folder on start
				if(activeFolder == title)accordionActiveId = i;
			}

			str += '<div class="hap-accordion-item" data-id="'+i+'" title="'+title+'"><span class="hap-accordion-item-title">'+title+'</span>'+
			'<div class="hap-accordion-item-content">'+
			'</div>'+
			'</div>';
		}

		playlistContent.html(str);		

	}

	//############################################//
	/* accordion */
	//############################################//

	function updatePlaylist(update_playlist_data, update_stats){

		if(update_playlist_data)playlistLength = playlistDataArr.length;

		if(usePlaylist){

			var i = 0, item, num, stat_arr = [], title_node;

			playlistContent.find('.hap-playlist-item').each(function(){
				
				item = $(this).attr('data-id', i);

				if(useStatistics && update_stats && !item.hasClass('hap-stats-set') && item.attr('data-media-id') != undefined){
					stat_arr.push({media_id: item.attr('data-media-id'), title: item.attr('data-safe-title') || '', artist: item.attr('data-safe-artist') || ''});
				}

				title_node = item.find('.hap-playlist-title');

				if(title_node.length){

					if(useNumbersInPlaylist){//append counter
						num = HAPUtils.formatNumber(i) + numberTitleSeparator;
						if(item.find('.hap-playlist-title-num').length){//if already exist
							item.find('.hap-playlist-title-num').html(num);
						}else{
							title_node.before($('<span class="hap-playlist-title-num">'+num+'</span>'));
						}
					}

				}

				if(update_playlist_data)playlistDataArr[i].id = i;

				i++;
			});

			if(useStatistics && update_stats && stat_arr.length)getStats('hap_all_count', null, stat_arr);

		}else{

			var i;
			for(i = 0;i < playlistLength; i++){
				playlistDataArr[i].id = i;
			}

		}
	}

	/*function updatePlaylistNumbers(){

		var i = 0, item, title_node, num;

		playlistContent.find('.hap-playlist-item').each(function(){

			item = $(this);
			title_node = item.find('.hap-playlist-title');

			num = HAPUtils.formatNumber(i) + numberTitleSeparator;
			if(item.find('.hap-playlist-title-num').length){//if already exist
				item.find('.hap-playlist-title-num').html(num);
			}else{
				title_node.before($('<span class="hap-playlist-title-num">'+num+'</span>'));
			}

			i++;

		});
	}*/

	function checkPlaylistScroll(elem){

		var axis = playlistScrollOrientation == 'horizontal' ? 'x' : 'y';

		elem.mCustomScrollbar({
			axis:axis,
			theme:settings.playlistScrollTheme,
			scrollInertia:0,
			mouseWheel:{ 
			 	normalizeDelta: true,
			 	deltaFactor: 50,
			 	preventDefault: true
			},
			keyboard:{enable:false},//sortableTracks will override this!
			advanced:{ autoExpandHorizontalScroll: true },
			callbacks:{
			    onOverflowYNone:function(){
			        elem.find('.mCSB_container').addClass('hap-mCSB_full');//hide scrollbar area if no scroll
			    }, 
			    onOverflowY:function(){
		            elem.find('.mCSB_container').removeClass('hap-mCSB_full');
		        },
		        onTotalScroll: function(){
			   
			    	if(loadMoreOnTotalScroll){
			    		if(nextPageToken && loadMoreType){
				    		if(!playlistTransitionOn){

				    			playlistTransitionOn = true;
				    			preloader.show();
				    			loadMoreProcess = true;
								playlistProcessData = [];
								if(loadMoreBtn)loadMoreBtn.css('opacity',0);

								if(loadMoreType == 'soundcloud'){
									resumeSoundCloud();
								}else if(loadMoreType == 'folder'){
									resumeFolderLoad();	
								}else if(loadMoreType == 'youtube'){
									if(!_HAPYoutubeLoader)createLoader('youtube');
									_HAPYoutubeLoader.resumeLoad(nextPageToken);
								}
							}
						}
					}else if(addMoreOnTotalScroll){
						if(!loadMoreProcess){
							addMore();
						}
					}
			    }
			}
		});

	}
	
	function endInit(){

		preloader.hide();
		playlistTransitionOn = false;

		if(!componentInited){
			componentInited = true;

			if(usePlaylistScroll && !playlistScrollInit){//was calling total scroll event (and calling add more) so we put it here
				checkPlaylistScroll(playlistInner);
				playlistScrollInit = true;
			}

			//global playlist options
			if(playlistContent.find('.hap-playlist-options').length){
				playlistData = playlistContent.find('.hap-playlist-options');
			
				if(playlistData.attr('data-add-more-num-results') != undefined && playlistData.attr('data-add-more-limit') != undefined && playlistData.attr('data-add-more-offset') != undefined && playlistData.attr("data-add-more-sort-order") != undefined){
					addMoreLimit = parseInt(playlistData.attr('data-add-more-limit'),10);
					addMoreNumResults = parseInt(playlistData.attr('data-add-more-num-results'),10);
					addMoreOffset = parseInt(playlistData.attr('data-add-more-offset'),10);
					addMoreSortOrder = playlistData.attr("data-add-more-sort-order");
					encryptMediaPaths = playlistData.attr("data-encrypt-media-paths");
				}

				if(playlistData.attr('data-thumb-global') != undefined && !HAPUtils.isEmpty(playlistData.attr('data-thumb-global'))){
					globalThumb = playlistData.attr('data-thumb-global');
				}
				if(playlistData.attr('data-start') != undefined && !HAPUtils.isEmpty(playlistData.attr('data-start'))){
					globalStartTime = Number(playlistData.attr('data-start'));
				}
				if(playlistData.attr('data-end') != undefined && !HAPUtils.isEmpty(playlistData.attr('data-end'))){
					globalEndTime = Number(playlistData.attr('data-end'));
				}
				if(playlistData.attr('data-playback-rate') != undefined && !HAPUtils.isEmpty(playlistData.attr('data-playback-rate'))){
					var pr = Number(playlistData.attr('data-playback-rate'));
					globalPlaybackRate = HAPUtils.valueLimit(pr, settings.playbackRateMin, settings.playbackRateMax);
				}
				if(playlistData.attr('data-add-more-on-total-scroll') != undefined){
					addMoreOnTotalScroll = true;
				}
				if(playlistData.attr('data-media-prefix-url') != undefined){
					mediaPrefixUrl = playlistData.attr('data-media-prefix-url');
				}
				//ads
				if(playlistData.attr('data-ad-pre') != undefined){
					globalAdPre = playlistData.attr('data-ad-pre').split(",").map(function(item){
					    return item.trim();
					});
					if(playlistData.attr('data-shuffle-ads') != undefined && globalAdPre.length>1)HAPUtils.shuffleArray(globalAdPre);
				}
				if(playlistData.attr('data-ad-mid') != undefined){
					globalAdMid = playlistData.attr('data-ad-mid').split(",").map(function(item){
					    return item.trim();
					});
					if(playlistData.attr('data-shuffle-ads') != undefined && globalAdMid.length>1)HAPUtils.shuffleArray(globalAdMid);
					if(playlistData.attr('data-ad-mid-interval') != undefined){
						globalAdMidInterval = playlistData.attr('data-ad-mid-interval');
					}
				}
				if(playlistData.attr('data-ad-end') != undefined){
					globalAdEnd = playlistData.attr('data-ad-end').split(",").map(function(item){
					    return item.trim();
					});
					if(playlistData.attr('data-shuffle-ads') != undefined && globalAdEnd.length>1)HAPUtils.shuffleArray(globalAdEnd);
				}

			}
			
		    setTimeout(function(){
				clearTimeout(this);doneResizing();
				wrapper.css('opacity',1);
			},50);

			setVolume(settings.volume);

			$(self).trigger('setupDone', {instance:self, instanceName:instanceName});

			if(accordionMode){

				lastOpenedAccordion = wrapper.find('.hap-accordion-item').eq(accordionActiveId);

	            wrapper.find('.hap-accordion-item-title').on('click', function(){
	                var parent = $(this).closest('.hap-accordion-item');

	                if(parent.hasClass('hap-accordion-item-opened')){
	                    parent.removeClass('hap-accordion-item-opened');
	                    var content = parent.find('.hap-accordion-item-content')[0]
	                    content.style.height = "0";
	                }else{
	                    //close other opened if new is clicked
	                    if(allowOnlyOneOpenedAccordion){
	                        if(lastOpenedAccordion && lastOpenedAccordion != parent){
	                            lastOpenedAccordion.removeClass('hap-accordion-item-opened');
	                            var content = lastOpenedAccordion.find('.hap-accordion-item-content')[0]
	                            content.style.height = "0";
	                        }
	                    }

	                    if(parent.hasClass('hap-accordion-item-inited')){
	                        var content = parent.find('.hap-accordion-item-content')[0]
	                        content.style.height = content.scrollHeight + "px";
	                    }else{
	                        if(accordionLoading)return false;
	                        accordionLoading = true;
	                        checkAccordion(parent); 
	                    }

	                    lastOpenedAccordion = parent.addClass('hap-accordion-item-opened');
	                    
	                }

	            });

			}

		}

		if(usePlaylist && playlistLength > 0){

			//fade thumbs
			var fadeArr = [];
			
			playlistContent.find('.hap-thumbimg:not(.hap-visible)').each(function(){
				fadeArr.push($(this));
			});

			var p=0, len = fadeArr.length, z;
			for(z=0;z<len;z++){
				setTimeout(function(){
				   clearTimeout(this);
				   fadeArr[p].addClass('hap-visible');
				   p++;
				},50 + ( z * 50 ));
			}

			if(!HAPUtils.isEmpty(sortOrder))self.sort(sortOrder);//sort on start

		}

		if(!autoInitActiveItem){//only after destroy playlist, not after add media
			if(!addTrackProcess && !loadMoreProcess && playlistLength > 0){
				var ai = settings.activeItem;
				if(ai > playlistLength-1)ai = playlistLength-1;
				if(ai>-1)_playlistManager.setCounter(ai, false);
				autoInitActiveItem = true;
			}
		}

		addTrackProcess = false;
		loadMoreProcess = false;

		if(loadMoreBtn){
			if(loadMoreOnTotalScroll){
				if(nextPageToken){
					loadMoreBtn.css('opacity',1);
				}else{
	        		loadMoreBtn.remove();
					loadMoreBtn = null;
					loadMoreOnTotalScroll = false;
	        	} 
	        }else if(addMoreOnTotalScroll){
	        	if(addMoreNumResults > 0){
					if(addMoreNumResults > addMoreOffset){
						loadMoreBtn.css('opacity',1);
					}else{
						loadMoreBtn.remove();
						loadMoreBtn = null;
						addMoreOnTotalScroll = false;
					}
				}else{
					loadMoreBtn.remove();
					loadMoreBtn = null;
				}
	        }else{
	        	loadMoreBtn.remove();
				loadMoreBtn = null;
	        }
        }

        $(self).trigger('playlistEndLoad', {instance:self, instanceName:instanceName});

        if(class_list.indexOf('hap-wall') > -1){

        	if(!_masonry){

        		if(typeof $.fn.masonry === 'undefined'){
        			alert('Link to masonry.pkgd.min.js file missing in head tag!')
        		}
        		if(typeof imagesLoaded !== 'function'){
        			alert('Link to imagesloaded.pkgd.min file missing in head tag!');
        		}

	        	_masonry = playlistContent.masonry({
				    itemSelector: '.hap-playlist-item',
				    columnWidth: '.hap-grid-sizer'
				});

				playlistContent.imagesLoaded( function() {
					_masonry.masonry('layout');
				});

			}else{//load more

				playlistContent.masonry('reloadItems');

			    playlistContent.imagesLoaded( function() {
					_masonry.masonry('layout');
				});

			}

			//append play btn to playlist items
			playlistContent.find('.hap-playlist-item-content').each(function(){
				var item = $(this);
				if(item.find('.hap-wall-preview').length == 0){
					item.find('.hap-playlist-thumb').after($('<div class="hap-wall-preview"></div>'));
				}
				if(item.find('.hap-wall-overlay').length == 0){
					item.find('.hap-playlist-thumb').after($('<div class="hap-wall-overlay"></div>'));
				}
			});	

		}

	}

	//############################################//
	/* hls */
	//############################################//

	function initHls(){

		hls = new Hls();

		hlsSupport = Hls.isSupported();
		hlsInited = true;

	    hls.on(Hls.Events.MEDIA_ATTACHED, function () {
	        //console.log("video and hls.js are now bound together !");
			hls.loadSource(mediaPath);
	    });

		hls.on(Hls.Events.ERROR, function (event, data) {
	    	if(data.fatal){
	        	switch(data.type) {
		        case Hls.ErrorTypes.NETWORK_ERROR:
			        // try to recover network error
			        console.log("fatal network error encountered, try to recover");
			        hls.startLoad();
			        break;
		        case Hls.ErrorTypes.MEDIA_ERROR:
			        console.log("fatal media error encountered, try to recover");
			        hls.recoverMediaError();
			        break;
		        default:
			        // cannot recover
			        hls.destroy();
			        break;
		        }
	        }
	    });

	}

	//############################################//
	/* audio player */
	//############################################//

	function initAudio(){
		//console.log('initAudio')

		if(mediaType == 'hls'){

			mediaPath = currMediaData.path;

			if(!hlsInited){

				if(typeof Hls === 'undefined'){
					
					var script = document.createElement('script');
					script.type = 'text/javascript';
					script.src = hls_js;
					script.onload = script.onreadystatechange = function() {
					    if(!this.readyState || this.readyState == 'complete'){
					      	initHls();
							initAudio();
					    }
					};
					script.onerror = function(){
						alert("Error loading " + this.src);
					}
					var tag = document.getElementsByTagName('script')[0];
					tag.parentNode.insertBefore(script, tag);

				}else{
					initHls();
					initAudio();
				}
				return;
			}

		}
		else if(mediaType == 'shoutcast' || mediaType == 'icecast' || mediaType == 'radiojar'){

			if(useCorsForAudio)mediaPath = settings.cors + currMediaData.path;
			else mediaPath = currMediaData.path;

			if(currMediaData.mountpoint)mediaPath += currMediaData.mountpoint;

			if(mediaType == 'shoutcast'){
				if(mediaPath.substring(mediaPath.length-2)!="/;")mediaPath+="/;";
			}

			if(!_HAPRadioData){	
				if(typeof HAPRadioData === 'undefined'){
					alert('link to radio.js file missing in head tag!');
				}

		      	_HAPRadioData = new HAPRadioData(settings, playerThumb);
		      	$(_HAPRadioData).on('HAPRadioData.DATA_READY', function(e,data){

					if(data.title)currMediaData.title = data.title;
					if(data.artist)currMediaData.artist = data.artist;
					if(data.thumb)currMediaData.thumb = data.thumb;

					setTitleAndThumb();

				})
				_HAPRadioData.getData(currMediaData);
				 
			}else{
				_HAPRadioData.getData(currMediaData);
			}
		}
		else{
			var i, key;
			for(i = 0; i < audioFormatOrder.length; i++){
				key = audioFormatOrder[i];
				if(key == 'mp3'){
					if(mp3Support && currMediaData.mp3){
						mediaPath = currMediaData.mp3;
						break;
					}
				}
				else if(key == 'aac'){
					if(aacSupport && currMediaData.aac){
						mediaPath = currMediaData.aac;
						break;
					}
				}
				else if(key == 'wav'){
					if(wavSupport && currMediaData.wav){
						mediaPath = currMediaData.wav;
						break;
					}
				}
			}

		} 


		

		if(!mediaPath){	
			alert('No required audio format supplied! Please add mp3 audio format to play!');
			return;
		}

		if(mediaPath.indexOf(bsf_match) != -1)mediaPath = atob(mediaPath.substr(6));

		if(mediaType == 'audio' && mediaPrefixUrl && !HAPUtils.relativePath(mediaPath))mediaPath = mediaPrefixUrl + mediaPath;

		audio.on("ended", function(){
			if(_HAPAdManager && _HAPAdManager.isAdEndOn())return;//we have double ended events (from hap and from admanager)

			if(useRange && rangeOn && !rangeSet && !_HAPRangeSlider.isDrag()){
				rangeSet = true;
				audioUp2Js.currentTime = range_a_time;
				audioUp2Js.play();
				return;
			}
			mediaEndHandler();	
		})
		.on('canplay', function(e){
		})
		.on('canplaythrough', function(e){
		})
		.on("loadedmetadata", function(){

			if(currMediaData.playbackRate == undefined)currMediaData.playbackRate = 1;
			audioUp2Js.playbackRate = currMediaData.playbackRate;
			if(_HAPPlaybackRateSlider)_HAPPlaybackRateSlider.setVisual(currMediaData.playbackRate);

			if(mediaType == 'audio'){//radio cannot be seeked
				if(settings.resumeTime){//from continous playback
					audioUp2Js.currentTime = settings.resumeTime;
					delete settings.resumeTime;
				}
				else if(currMediaData.start)audioUp2Js.currentTime = currMediaData.start;
			}

			//set visible duration
			var d = audioUp2Js.duration;
			if(HAPUtils.isNumber(d))mediaTimeTotal.html(HAPUtils.formatTime(d));

		})
		.on("play", function(){
			if(mediaType != 'shoutcast' && mediaType != 'icecast' && mediaType != 'radiojar' && mediaType != 'hls'){
				if(startEventDone)playHandler();
			}else{
				playHandler();
			}
		})
		.on("pause", function(){
			pauseHandler();
		})
		.on("seeked", function(){
			if(useRange && rangeOn){
				setTimeout(function(){
					clearTimeout(this);
					rangeSet = false;
				},500);
			}
		})	
		.on("error", function(e){
			$(self).trigger('soundError', {instance:self, instanceName:instanceName, counter:mediaCounter, error:e});
		});


		if(mediaType == 'hls'){

			if(window.location.protocol == 'file:'){
				alert('Playing HLS requires server connection!');
				return false;
			}

			if(hlsSupport){
				hls.attachMedia(audioUp2Js);
			}else if(audioUp2Js.canPlayType('application/vnd.apple.mpegurl')){//ios backup
			    audioUp2Js.src = mediaPath;
			}else if(currMediaData.mp3){
				audioUp2Js.src = currMediaData.mp3;
			}else{
				alert("This browser or device does not support HLS extension. Please use mp3 audio for playback!");
				return;
			}

			if(autoPlay){
				var promise = audioUp2Js.play();
				if (promise !== undefined) {
					promise.then(function(){
				    }).catch(function(error){
				    });
				}
			}

		}else{

			audioUp2Js.src = mediaPath;

			if(autoPlay){
				var promise = audioUp2Js.play();
				if (promise !== undefined) {
					promise.then(function(){
				    }).catch(function(error){
				    });
				}
			}

		}

		audioUp2Js.volume = volume;
		autoPlay = true;

		//10.3.2020 - added this here because there is a lot of delay on ios
		if(mediaType != 'shoutcast' && mediaType != 'icecast' && mediaType != 'radiojar'){
			if(dataIntervalID) clearInterval(dataIntervalID);
			dataIntervalID = setInterval(trackProgress, dataInterval);
		}
	
	}

	//############################################//
	/*  */
	//############################################//

	function trackProgress(){
		if(seekBarDown)return;//for performance
		if(pauseAudioDuringAds && _HAPAdManager && _HAPAdManager.isAdOn()) return;

		var loadPercent;

		if(mediaType == 'youtube'){

			if(youtubePlayer){
				var t = youtubePlayer.getCurrentTime(), d = youtubePlayer.getDuration();	
				loadPercent = youtubePlayer.getVideoLoadedFraction();
			}

		}else if(mediaType == 'audio'){

			if(audioUp2Js){
				var t = audioUp2Js.currentTime, d = audioUp2Js.duration;

				if(HAPUtils.isNumber(t) && HAPUtils.isNumber(d)){
					
					if(typeof audioUp2Js.buffered !== 'undefined' && audioUp2Js.buffered.length != 0) {
						try{
							var bufferedEnd = audioUp2Js.buffered.end(audioUp2Js.buffered.length - 1);
						}catch(error){}
						if(!isNaN(bufferedEnd)){
							var loadPercent = bufferedEnd / d;
						}
					}
				}
			}

		}

		if(HAPUtils.isNumber(t) && HAPUtils.isNumber(d)){

			if(!startEventDone){
				if(t > 0){
					startEventDone = true;
					playHandler();
				}
			}

			if(!circleSeekbar){
				progressLevel.width((t / d) * seekBarSize);
				if(loadPercent)loadLevel.width(loadPercent * seekBarSize);	
			}else{
				_HAPCirclePlayer.drawSeekbar(loadPercent,t,d);
			}

			mediaTimeCurrent.html(HAPUtils.formatTime(t));
			mediaTimeTotal.html(HAPUtils.formatTime(d));

			if(!seekBarDown){
				if(currMediaData.end){
					if(t >= currMediaData.end){
						mediaEndHandler();
						return;
					}
				}
			}

			if(useRange && rangeOn && !rangeSet && !_HAPRangeSlider.isDrag()){
				if(parseInt(t,10) < parseInt(range_a_time,10) || t > range_b_time){
					rangeSet = true;
					self.seek(range_a_time);
					return;
				}
			}

		}
	}

	function playHandler(){

		if(!mediaStarted){
			mediaStarted = true;	

			seekBarSize = progressBg.width();

			$(self).trigger("soundStart", {instance:self, instanceName:instanceName, data:currMediaData, audio: audioUp2Js});

			if(useGa && typeof ga !== "undefined"){
				ga('send', {
			        hitType: 'event',
			        eventCategory: 'Modern audio player: ' + instanceName,
			        eventAction: 'started',
			        eventLabel: 'title: ' + getTitle(currMediaData, true),
			        nonInteraction: true
			    });
			}

			//############################################//
			/* ads */
			//############################################//
			if(currMediaData.adMid){
				_HAPAdManager.adMidStartHandler();
			}
			//############################################//
			/* ads */
			//############################################//

		}

		//############################################//
		/* ads */
		//############################################//
		if(currMediaData.adMid){
			_HAPAdManager.adMidPlayHandler();
		}
		//############################################//
		/* ads */
		//############################################//

		if(settings.togglePlaybackOnMultipleInstances){
			if(hap_mediaArr.length > 1){
				var i, len = hap_mediaArr.length;
				for(i=0;i<len;i++){
					if(self != hap_mediaArr[i].inst){
						hap_mediaArr[i].inst.pauseMedia();
					}
				}
			}
		}

		playbackToggle.find('.hap-btn-play').hide();
		playbackToggle.find('.hap-btn-pause').show();

		mediaPlaying = true;
		$(self).trigger("soundPlay", {instance:self, instanceName:instanceName, data:currMediaData, audio: audioUp2Js});

		if(useGa && typeof ga !== "undefined"){
			ga('send', {
		        hitType: 'event',
		        eventCategory: 'Modern audio player: ' + instanceName,
		        eventAction: 'played',
		        eventLabel: 'title: ' + getTitle(currMediaData, true) + ' | time: ' + Math.round(self.getCurrentTime()),
		        nonInteraction: true
		    });
		}
	}

	function pauseHandler(){

		playbackToggle.find('.hap-btn-play').show();
		playbackToggle.find('.hap-btn-pause').hide();

		mediaPlaying = false;
		$(self).trigger("soundPause", {instance:self, instanceName:instanceName, data:currMediaData, audio: audioUp2Js});

		if(useGa && typeof ga !== "undefined"){
			ga('send', {
		        hitType: 'event',
		        eventCategory: 'Modern audio player: ' + instanceName,
		        eventAction: 'paused',
		        eventLabel: 'title: ' + getTitle(currMediaData, true) + ' | time: ' + Math.round(self.getCurrentTime()),
		        nonInteraction: true
		    });
		}
	}

	function mediaEndHandler(){

		if(_HAPAdManager)_HAPAdManager.clearAdMidTimeout();

		$(self).trigger('soundEnd', {instance:self, instanceName:instanceName, data:currMediaData, audio: audioUp2Js});

		if(useStatistics && activePlaylistItem)getStats('hap_finish_count', activePlaylistItem);

		if(useGa && typeof ga !== "undefined"){
			ga('send', {
		        hitType: 'event',
		        eventCategory: 'Modern audio player: ' + instanceName,
		        eventAction: 'ended',
		        eventLabel: 'title: ' + getTitle(currMediaData, true),
		        nonInteraction: true
		    });
		}

		if(currMediaData.adEnd){

			//############################################//
			/* ads */
			//############################################//

			_HAPAdManager.setAdEnd();

			//############################################//
			/* ads */
			//############################################//

		}else{
			audioEndedAction();
		}

	}

	function audioEndedAction(reset_src){
		console.log('audioEndedAction')

		var st = currMediaData.start || 0;

		if(loopState == 'single'){

			if(!reset_src)beforeSongAdvanceAction();//only is main media plays, not ad

			if(mediaType == 'youtube'){
				if(youtubePlayer){
					youtubePlayer.seekTo(st);
					youtubePlayer.playVideo();
				}
			}else{
				if(reset_src)audioUp2Js.src = mediaPath;//if coming from end ad
				audioUp2Js.currentTime = st;
				audioUp2Js.play();
			}

		}else if(loopState == 'off' || loopState == 'playlist'){
			self.nextMedia();
		}

	}

	function cleanMedia(){

		if(dataIntervalID) clearInterval(dataIntervalID);

		//############################################//
		/* ads */
		//############################################//

		if(_HAPAdManager){

			_HAPAdManager.cleanAds();

			progressLevel.removeClass('hap-ad-progress-level');
			mediaTimeAd.hide();

		}

		//############################################//
		/* ads */
		//############################################//

		if(mediaType == 'shoutcast' || mediaType == 'icecast' || mediaType == 'radiojar'){
			_HAPRadioData.destroy();
		}
		else if(mediaType == 'hls'){
			if(hlsSupport)hls.detachMedia();
		}
		else if(mediaType == 'youtube'){
			if(youtubePlayer){
				youtubeEndFix = true;
				youtubeHolder.hide();
				if(youtubeReady)youtubePlayer.stopVideo();
				youtubeStarted = false;
			}
		}
		if(audioUp2Js){
			audioUp2Js.pause();
			audioUp2Js.src = '';
		}
		if(audio)audio.off('ended canplay canplaythrough loadedmetadata pause play error timeupdate seeked');

	    mediaTimeCurrent.html('00:00');
		mediaTimeTotal.html('00:00');
	    playerTitle.html('');
		playerArtist.html('');
		progressLevel.width(0);
	   	loadLevel.width(0);

		mediaType = null;
		mediaPlaying = false;
		mediaStarted = false;
		addTrackPlayit = false;

		if(useRange)self.resetRange();

		playbackToggle.find('.hap-btn-play').show();
		playbackToggle.find('.hap-btn-pause').hide();

		startEventDone = false;

		if(useTitleScroll && _HAPTextScroller)_HAPTextScroller.deactivate();
		
	}

	function destroyMedia(){
		if(!componentInited || !mediaType) return;
		if(activePlaylistItem)enableActivePlaylistItem();
		cleanMedia();
		_playlistManager.reSetCounter();
	}
	
	function destroyPlaylist(){

		if(typeof $.fn.masonry !== "undefined" && _masonry)playlistContent.masonry('destroy');

		if(mediaType){
			cleanMedia();
			if(activePlaylistItem)enableActivePlaylistItem();
		}

		playlistContent.empty();

		lastPlaylist = null;
		addTrackProcess = false;
		playlistFirstInit = false;
		autoInitActiveItem = false;
		playlistLength = 0;
		playlistProcessCounter = -1;
		playlistDataArr = [];
		playlistProcessData = [];
		playlistProcessDataUrl = [];
		_playlistManager.reSetCounter();

		playlistData = null;
		globalThumb = null;
		globalStartTime = null;
		globalEndTime = null;
		globalPlaybackRate = null;
		globalAdPre = null;
		globalAdMid = null;
		globalAdMidInterval = null;
		globalAdEnd = null;
		addMoreOnTotalScroll = false;
		mediaPrefixUrl = null;

		playerThumb.css('backgroundImage','none').css('opacity',0);

		loadMoreProcess = false;
		nextPageToken = null;
		loadMoreType = null;
		loadMoreOnTotalScroll = false;

		addMoreLimit = null;
		addMoreNumResults = null;
		addMoreOffset = null;
		addMoreSortOrder = null;
		encryptMediaPaths = null;

		//reset sort because tracks are loaded with no sort. TODO: Do we keep last sort on new playlist load?
		sortOrder = "";
		sortAlpha.find('.hap-btn-sort-alpha-up').hide();
		sortAlpha.find('.hap-btn-sort-alpha-down').show();

		//search
		searchFilter.val('');
		playlistFilterMsg.hide();

		$(self).trigger('destroyPlaylist', {instance:self, instanceName:instanceName});

	}

	function beforeSongAdvanceAction(){
		//before we move to next song
		if(useContinousPlayback && continousPlaybackTrackAllSongs)playlistDataArr[mediaCounter].data.start = self.getCurrentTime();
		if(useStatistics && activePlaylistItem)getStats('hap_play_count', activePlaylistItem);
	}

	window.onbeforeunload = function(e){
        if(useStatistics && activePlaylistItem)getStats('hap_play_count', activePlaylistItem);
    };

	//############################################//
	/* resize */
	//############################################//
	
	var resize_event = 'onorientationchange' in window ? 'orientationchange' : 'resize';
	_window.on(resize_event,function() {
		 if(!componentInited) return false;
		 if(windowResizeTimeoutID) clearTimeout(windowResizeTimeoutID);
		 windowResizeTimeoutID = setTimeout(doneResizing, windowResizeTimeout);
	});
	
	function doneResizing(){

	    seekBarSize = progressBg.width();
	    if(_HAPAdManager)_HAPAdManager.setSeekBarSize(seekBarSize);

	    if(useTitleScroll && _HAPTextScroller){
		   _HAPTextScroller.checkSize();
	    }

	    if(useRange && typeof range_a_time !== 'undefined' && typeof range_b_time !== 'undefined'){
			var d = self.getDuration();
			if(typeof d !== 'undefined'){

				if(range_handle_a)range_handle_a.css('left',((range_a_time / d) * seekBarSize)+'px');
				if(range_handle_b)range_handle_b.css('left',((range_b_time / d) * seekBarSize)+'px');
			}
		}

		tooltip.hide();

	}

	//############################################//
	/* misc */
	//############################################//
	
	function getTitle(data, clean){//get title from data
		var title;

		if(data.artist && !HAPUtils.isEmpty(data.artist) && data.title && !HAPUtils.isEmpty(data.title)){
			if(clean)title = data.artist + ' - ' + data.title;
			else title = data.artist + artistTitleSeparator + data.title;
		}
		else if(data.title && !HAPUtils.isEmpty(data.title)){
			title = data.title;
		}
		else if(data.artist && !HAPUtils.isEmpty(data.artist)){
			title = data.artist;
		}

		return title;
	}
	
	function setTitleAndThumb(){

		//title, artist
		
		if(currMediaData.title)playerTitle.html(currMediaData.title);
		if(currMediaData.artist)playerArtist.html(currMediaData.artist);
		if(currMediaData.description)wrapper.find('.hap-player-desc').html(currMediaData.description);

		if(useTitleScroll && _HAPTextScroller){
			_HAPTextScroller.input(getTitle(currMediaData));
		}

		if(playerThumbInited && globalThumb)return;

		//thumb
		if(playerThumb.length){
			var next_thumb = currMediaData.thumb || currMediaData.thumbDefault;
			if(!next_thumb)return;
			setThumb(next_thumb);
		}
	}

	function setThumb(next_thumb){

		if(mediaPrefixUrl && !HAPUtils.relativePath(next_thumb))next_thumb = mediaPrefixUrl + next_thumb;

		if(currMediaData.origtype == 'soundcloud'){
			if(settings.soundCloudThumbQuality)next_thumb = next_thumb.replace('large.jpg', settings.soundCloudThumbQuality);
		}

		next_thumb = encodeURI(next_thumb);//spaces in url

		if(currMediaData.title)playerThumb.attr('aria-label', currMediaData.title);
		else playerThumb.attr('aria-label','');

		if(playerThumbInited){
			playerThumb.css('opacity',0);
			setTimeout(function(){
				playerThumb.css('backgroundImage','url('+next_thumb+')').css('opacity',1);
			},300);
		}else{
			playerThumb.css('backgroundImage','url('+next_thumb+')').css('opacity',1);
		}
		playerThumbInited = true;


		//preset mods
		if(class_list.indexOf('hap-metalic') > -1){

            wrapper.find('.hap-player-image').fadeOut(300, function(){
                $(this).css('backgroundImage','url('+next_thumb+')').fadeIn(300);
            });

        }
        else if(class_list.indexOf('hap-poster') > -1){

            wrapper.find('.hap-player-image').fadeOut(300, function(){
                $(this).css('backgroundImage','url('+next_thumb+')').fadeIn(300);
            });

        }

	}

	function fetchArtwork(){

		//https://stackoverflow.com/questions/13382208/getting-bigger-artwork-images-from-itunes-web-search

		var artist = currMediaData.artist || '', title = currMediaData.title || '',

		url = "https://itunes.apple.com/search?type=jsonp&term==" + encodeURI(artist) + "-" + encodeURI(title) + "&media=music&limit=1";

		$.ajax({
			url: url,
			dataType: "jsonp"
		}).done(function(response) {
		
			if(response.results[0] && response.results[0].artworkUrl100){

				//calculate thumb size
				var w = playerThumb.width(),
				s = HAPUtils.closestNumber(settings.artworkSize, w),
				size = s+'x'+s

				var thumb = response.results[0].artworkUrl100.replace('100x100', size);
				playlistDataArr[mediaCounter].data.thumb = thumb;
				setThumb(thumb);
			}

		}).fail(function(jqXHR, textStatus, errorThrown) {
			console.log(jqXHR, textStatus, errorThrown);
		});

	}

	//############################################//
	/* statistics */
	//############################################//

	if(useStatistics){

	    wrapper.on('click','.hap-like-count',function(e){
	    	if(!componentInited) return false;
			if(!mediaType) return false;

			var p = $(this).closest('.hap-playlist-item');
	        getStats('hap_like_count', p);
	    });

	    wrapper.on('click','.hap-download-count:not(.hap-no-download)',function(e){
	    	if(!componentInited) return false;
			if(!mediaType) return false;
			
			var p = $(this).closest('.hap-playlist-item');
			getStats('hap_download_count', p);

	    });

	}

	function getStats(action, p, arr){

		if(window.location.protocol == 'file:'){
			return;
		}

		if(action == 'hap_all_count'){

			var postData = [
		        {name: 'action', value: action},
		        {name: 'data', value: JSON.stringify(arr)},
		    ];

		}
		else if(action == 'hap_play_count'){

			var postData = [
		        {name: 'action', value: action},
		        {name: 'playlist_id', value: playlistId},
		        {name: 'media_id', value: p.attr('data-media-id')},
		        {name: 'title', value: p.attr('data-safe-title') || ''},
		        {name: 'artist', value: p.attr('data-safe-artist') || ''},
		        {name: 'currentTime', value: audioUp2Js.currentTime},
		        {name: 'duration', value: audioUp2Js.duration},
		    ];

		}
		else if(action == 'hap_download_count' || action == 'hap_like_count' || action == 'hap_finish_count'){

			var postData = [
		        {name: 'action', value: action},
		        {name: 'playlist_id', value: playlistId},
		        {name: 'media_id', value: p.attr('data-media-id')},
		        {name: 'title', value: p.attr('data-safe-title') || ''},
		        {name: 'artist', value: p.attr('data-safe-artist') || ''},
		    ];

		}
		console.log(postData, action)

		var	url = settings.ajax_url;

	    $.ajax({
	        url: url,
	        type: 'post',
	        data: postData,
	        dataType: "json"
	    }).done(function(response){
	    	console.log('response ', response)

	    	if(response){

	    		if(action == 'hap_like_count' && response.c_like){
	    			p.find('.hap-like-count span').html(HAPUtils.nFormatter(parseInt(response.c_like,10), 1));
	    		}
	    		else if(action == 'hap_download_count' && response.c_download){
	    			p.find('.hap-download-count span').html(HAPUtils.nFormatter(parseInt(response.c_download,10), 1));
	    		}
	    		else if(action == 'hap_play_count' && response.c_play){
	    			p.find('.hap-play-count span').html(HAPUtils.nFormatter(parseInt(response.c_play,10), 1));
	    		}
	    		else if(action == 'hap_all_count'){
	    			var i, len = response.length, obj, item;
	    			for(i=0;i<len;i++){
	    				obj = response[i];

	    				if(obj.title && obj.artist){
	    					item = playlistContent.find('.hap-playlist-item[data-media-id="'+obj.media_id+'"][data-safe-title="'+obj.title+'"][data-safe-artist="'+obj.artist+'"]').addClass('hap-stats-set');
	    				}else if(obj.title){
	    					item = playlistContent.find('.hap-playlist-item[data-media-id="'+obj.media_id+'"][data-safe-title="'+obj.title+'"]').addClass('hap-stats-set');
	    				}else if(obj.artist){
	    					item = playlistContent.find('.hap-playlist-item[data-media-id="'+obj.media_id+'"][data-safe-artist="'+obj.artist+'"]').addClass('hap-stats-set');
	    				}	
						
	    				item.find('.hap-like-count span').html(HAPUtils.nFormatter(parseInt(obj.c_like,10), 1));
	    				item.find('.hap-download-count span').html(HAPUtils.nFormatter(parseInt(obj.c_download,10), 1));
	    				item.find('.hap-play-count span').html(HAPUtils.nFormatter(parseInt(obj.c_play,10), 1));
	    			}
	    		}

		    }
		        
	    }).fail(function(jqXHR, textStatus, errorThrown) {
	        console.log('Error getStats: ' + jqXHR.responseText, textStatus, errorThrown);
	    }); 

	}


	
	
	//############################################//
	/* api */
	//############################################//
	
	function log(s){
		var h = document.createElement("H1");
	    var t = document.createTextNode(s);
	    h.style.color = '#ffffff';
	    h.appendChild(t);
	    document.body.appendChild(h);
	}

	this.playMedia = function() {
		if(!componentInited) return false;
		if(!mediaType) return false;
		if(mediaPlaying) return false;

		if(mediaType == 'youtube'){
			if(youtubePlayer && youtubeReady)youtubePlayer.playVideo();
		}else{
			if(audioUp2Js){
				if(mediaType == 'shoutcast' || mediaType == 'icecast' || mediaType == 'radiojar'){
		    		audioUp2Js.load();
		    	}
				var promise = audioUp2Js.play();
				if (promise !== undefined) {
					promise.then(function(){
				    }).catch(function(error){
				    });
				}
			}
		}
	}
	this.pauseMedia = function() {	
		if(!componentInited) return false;
		if(!mediaType) return false;
		if(!mediaPlaying) return false;

		if(mediaType == 'youtube'){
			if(youtubePlayer && youtubeReady)youtubePlayer.pauseVideo();
		}else{
			if(audioUp2Js)audioUp2Js.pause();
		}
	}	
	this.checkMedia = function(a) {	
		if(!componentInited) return false;
		if(!mediaType) return false;
		var action = a.toLowerCase();
		if(mediaPlaying){
			if(action=='pause'){
				if(mediaType == 'youtube'){
					if(youtubePlayer && youtubeReady)youtubePlayer.pauseVideo();
				}else{
					if(audioUp2Js)audioUp2Js.pause();
				}
			}
		}
	}	
	this.togglePlayback = function(){
		if(!componentInited) return false;
		if(!mediaType) return false;
		if(_HAPAdManager)_HAPAdManager.clearAdMidTimeout();

		if(_HAPAdManager && _HAPAdManager.isAdMidOn()){

			//############################################//
			/* ads */
			//############################################//
		
			_HAPAdManager.toggleAdMidAudio();

			//############################################//
			/* ads */
			//############################################//

		}else{

			if(mediaType == 'youtube'){
				if(youtubePlayer && youtubeReady){
				    var player_state = youtubePlayer.getPlayerState();
				    
				    if(player_state == 1){//playing
						youtubePlayer.pauseVideo();
					}else if(player_state == 2){//paused
						youtubePlayer.playVideo();
					}else if(player_state == -1 || player_state == 5 || player_state == 0){//unstarted, cued, ended
						youtubePlayer.playVideo();
					}
				}
			}else{
				if(audioUp2Js){

				    if(audioUp2Js.paused) {
				    	if(mediaType == 'shoutcast' || mediaType == 'icecast' || mediaType == 'radiojar'){
				    		audioUp2Js.load();
				    	}
					    var promise = audioUp2Js.play();
						if (promise !== undefined) {
							promise.then(function(){
								//############################################//
								/* ads */
								//############################################//
								if(_HAPAdManager && _HAPAdManager.isAdPreOn()){
									playbackToggle.find('.hap-btn-play').hide();
									playbackToggle.find('.hap-btn-pause').show();
								}
								//############################################//
								/* ads */
								//############################################//
						    }).catch(function(error){
						    });
						}
				    }else{
					    audioUp2Js.pause();
					    //############################################//
						/* ads */
						//############################################//
					    if(_HAPAdManager && _HAPAdManager.isAdPreOn()){
							playbackToggle.find('.hap-btn-play').show();
							playbackToggle.find('.hap-btn-pause').hide();
						}
						//############################################//
						/* ads */
						//############################################//
				    }
				}
			}

		}
	}
	this.nextMedia = function() {
		if(!componentInited) return false;
		if(playlistLength == 0) return;
		beforeSongAdvanceAction();
		_playlistManager.advanceHandler(1, true);
	}
	this.previousMedia = function() {
		if(!componentInited) return false;
		if(playlistLength == 0) return;
		beforeSongAdvanceAction();
		_playlistManager.advanceHandler(-1, true);
	}
	this.loadMedia = function(format, value, title, artist) {
		if(!componentInited) return false;
		if(playlistTransitionOn) return false;
		if(playlistLength == 0) return;
				
		var index = -1;

		if(format == 'title'){

			var i, found;
			for(i = 0;i < playlistLength; i++){
				if(value == playlistDataArr[i].data.title){
					index = i;
					found = true;
					break;	
				}
			}

			if(!found){
				alert('No media with title "'+value+'" to load! LoadMedia failed.');
				return false;
			}
			
		}else if(format == 'counter'){
			
			if(value<0){
				alert('Invalid track number. Track number  "' + value + '" doesnt exist. LoadMedia failed.');
				return false;
			}
			else if(value > playlistLength-1){
				alert('Invalid track number. Track number  "' + value + '" doesnt exist. LoadMedia failed.');
				return false;
			}
			else{
				index = value;
			}
			
		}else if(format == 'id'){//data-media-id

			var pi = playlistContent.find('.hap-playlist-item[data-media-id='+value+']');
			
			if(pi.length == 0){
				alert('No media with media ID '+value+' to load! LoadMedia failed.');
				return false;
			}

			index = pi.attr('data-id');


		}else if(format == 'id-title'){//data-media-id and title, artist (we use this if playlist is grouped source and all items can have same media-id)

			if(title && artist){
				var pi = playlistContent.find('.hap-playlist-item[data-media-id="'+value+'"][title="'+title+'"][data-artist="'+artist+'"]');
			}else if(title){
				var pi = playlistContent.find('.hap-playlist-item[data-media-id="'+value+'"][title="'+title+'"]');
			}else if(artist){
				var pi = playlistContent.find('.hap-playlist-item[data-media-id="'+value+'"][data-artist="'+artist+'"]');
			}	
			
			if(pi.length == 0){
				alert('No media with media ID '+value+' to load! LoadMedia failed.');
				return false;
			}

			index = pi.attr('data-id');

		}else{
			console.log('loadMedia function requires format parameter!');
			return false;
		}

		beforeSongAdvanceAction();
		
		_playlistManager.processPlaylistRequest(index);
	}

	this.loadMore = function(){
		if(!componentInited) return false;
		if(!loadMoreType) return false;
		if(!nextPageToken) return false;
		if(playlistTransitionOn)return false;

		playlistTransitionOn = true;
		preloader.show();
		loadMoreProcess = true;
		playlistProcessData = [];

		if(loadMoreType == 'soundcloud'){
			resumeSoundCloud();
		}else if(loadMoreType == 'folder'){
			resumeFolderLoad();
		}else if(loadMoreType == 'youtube'){
			if(!_HAPYoutubeLoader)createLoader('youtube');
			_HAPYoutubeLoader.resumeLoad(nextPageToken);
		}
	}
	this.setLoadMore = function(value){
		loadMoreOnTotalScroll = value;
	}
	this.endLoadMore = function(){
		playlistTransitionOn = false;
		playerLoader.hide();
		loadMoreProcess = false;
	}
	function addMore(){//load more from database

		preloader.show();
		loadMoreProcess = true;
		if(loadMoreBtn)loadMoreBtn.css('opacity',0);

	    var postData = [
	        {name: "action", value: "hap_add_more"},
	        {name: "playlist_id", value: playlistId},
	        {name: "addMoreOffset", value: addMoreOffset},
	        {name: "addMoreLimit", value: addMoreLimit},
	        {name: "addMoreSortOrder", value: addMoreSortOrder},
	        {name: "encryptMediaPaths", value: encryptMediaPaths},
	    ];

	    $.ajax({
	        url: settings.ajax_url,
	        type: "post",
	        data: postData,
	        dataType: "json",   
	    }).done(function(response){
	    	//console.log(response)

	    	if(playlistData && playlistData.attr('data-add-more-offset') != undefined){
				addMoreOffset = parseInt(playlistData.attr('data-add-more-offset'),10) + addMoreLimit;
				playlistData.attr('data-add-more-offset', addMoreOffset);
			}

	        self.addTrack(response);

	    }).fail(function(jqXHR, textStatus, errorThrown) {
	        console.log(jqXHR, textStatus, errorThrown);
			self.endLoadMore();
	    }); 

	}
	this.addMore = function(){
		if(!componentInited) return false;
		if(playlistTransitionOn)return false;
		if(addMoreOnTotalScroll){
			if(!loadMoreProcess){
				addMore();
			}
		}
	}
	this.setAddMore = function(value){
		addMoreOnTotalScroll = value;
	}
	
	this.loadPlaylist = function(id) {
		if(!componentInited) return false;
		if(playlistTransitionOn) return false;
		if(typeof id === 'undefined' || HAPUtils.isEmpty(id)){
			alert('loadPlaylist method requires id parameter. loadPlaylist failed.');
			return false;
		}
		if(settings.activePlaylist == id)return false;//playlist already loaded!
		setPlaylist(id);
	}
	this.addTrack = function(track, playit, position) {
		if(!componentInited || playlistTransitionOn) return false;

		if(typeof track === 'undefined'){
			alert('addTrack method requires track parameter. AddTrack failed.');
			return false;
		}
		addTrackPlayit = false;
		if(typeof playit !== 'undefined'){
			addTrackPlayit = playit;
		}

		insertPosition = position;
		endInsert = false;
		addTrackProcess = true;
		
		if(lastPlaylist){
			if(typeof insertPosition !== 'undefined'){
				if(insertPosition < 0){
					alert('Invalid position to insert track to. Position number "' + position + '" doesnt exist. AddTrack failed.');
					return false;
				}
				else if(insertPosition > playlistLength){
					alert('Invalid position to insert track to. Position number "' + position + '" doesnt exist. AddTrack failed.');
					return false;
				}
				else if(insertPosition == playlistLength){
					endInsert = true;
				}
			}else{
				endInsert = true;
				insertPosition = playlistLength;	
			}
		}else{//first time create playlist from addTrack method
			if(typeof insertPosition !== 'undefined'){
				if(insertPosition != 0){
					alert('Invalid position to insert track to. Position number "' + position + '" doesnt exist. AddTrack failed.');
					return false;
				}
			}else{
				insertPosition = 0;
			}
			endInsert = true;
		}
		
		playlistTransitionOn = true;
		preloader.show();
		
		playlistProcessCounter = -1;
		playlistProcessData = [];
		playlistProcessDataUrl = [];

		if(Array.isArray(track))playlistProcessDataUrl = track;
		else playlistProcessDataUrl.push(track);

		playlistLength = playlistProcessDataUrl.length;
	
		playlistFirstInit = false;
		if(!lastPlaylist)playlistFirstInit = true;
		lastPlaylist = playlistContent;
		
		checkProcessCounter();

	}

	this.inputAudio = function(data){
		if(!componentInited) return false;
		if(playlistTransitionOn) return false;

		if(typeof data === 'undefined'){
			alert('inputAudio method requires data parameter. inputAudio failed.');
			return false;
		}

		//enable active item, if exist, but dont dispose current playlist
		if(activePlaylistItem)enableActivePlaylistItem();
		if(playlistLength > 0)_playlistManager.reSetCounter();
		
		currMediaData = data;
		mediaType = data.type;

		if(nonProcessMedia.indexOf(mediaType) == -1){
			alert('inputAudio method supports tracks that dont require processing: ' + nonProcessMedia);
			return false;
		}

		if(mediaType != 'shoutcast' && mediaType != 'icecast' && mediaType != 'radiojar'){
			setTitleAndThumb();
			if(useMediaSession)updateMetadata();
		}

		autoPlay = true;

		initAudio();
	}
	
	this.removeTrack = function(format, value) {
		if(!componentInited || playlistTransitionOn) return false;
		if(playlistLength == 0) return;

		var track, found, title;

		if(format == 'title'){

			var i;
			for(i = 0;i < playlistLength; i++){
				if(value == playlistDataArr[i].data.title){
					track = playlistContent.children('.hap-playlist-item').eq(i);
					found = true;
					break;	
				}
			}

			if(!found){
				alert('Track with title "' + value + '" doesnt exist. RemoveTrack failed.');
				return false;	
			}
			
		}else if(format == 'counter'){
			
			value = parseInt(value,10);
			
			if(value < 0 || value > playlistLength-1){
				alert('Track number  "' + value + '" doesnt exist. RemoveTrack failed.');
				return false;
			}

			track = playlistContent.find('.hap-playlist-item').eq(value);
			
		}else if(format == 'id'){//data-media-id

			track = playlistContent.find('.hap-playlist-item[data-media-id='+value+']');

			if(track.length == 0){
				alert('Track with media id "' + value + '" doesnt exist. RemoveTrack failed.');
				return false;	
			}

		}else{
			alert('removeTrack method failed.');
			return false;
		}


		//remove track

		track.remove();
		playlistDataArr.splice(parseInt(track.attr('data-id'),10),1);
		
		updatePlaylist(true);	
		
		if(playlistLength > 0){
			
			var current_counter = _playlistManager.getCounter();
			if(value == current_counter){//remove number equal to current counter
				cleanMedia();	
				_playlistManager.setPlaylistItems(playlistLength);//counter resets to -1
			}else{
				_playlistManager.setPlaylistItems(playlistLength, false);
				if(value < current_counter){//remove number less than current counter
					_playlistManager.reSetCounter(_playlistManager.getCounter()-1);//if we removed media before current playing media, descrease counter!	
				}else{//remove number larger than current counter, current counter doesnt change
				}
			}

		}else{//we removed last track in playlist
			destroyPlaylist();	
		}

	}

	this.sort = function(order) {
		if(!componentInited) return false;
		if(playlistLength < 3) return false;
	
		if(typeof order === 'undefined'){
			console.log('Sort method requires order parameter. Sort method failed.');
			return false;
		}

		if(sortOrder == v) return false;

		var items = playlistContent.children('.hap-playlist-item'), len = items.length, v = order.toLowerCase();

		if(v == 'title-asc'){

			HAPUtils.keysrt2(playlistDataArr, 'data', 'title');

			var i, arr = [];
			for(i = 0;i < playlistLength; i++){
				arr.push(playlistDataArr[i].id);
			}

			sortAlpha.find('.hap-btn-sort-alpha-up').hide();
			sortAlpha.find('.hap-btn-sort-alpha-down').show();
		
		}
		else if(v == 'title-desc'){

			HAPUtils.keysrt2(playlistDataArr, 'data', 'title', true);

			var i, arr = [];
			for(i = 0;i < playlistLength; i++){
				arr.push(playlistDataArr[i].id);
			}

			sortAlpha.find('.hap-btn-sort-alpha-up').show();
			sortAlpha.find('.hap-btn-sort-alpha-down').hide();

		}
		else if(v == 'random'){

			var arr = HAPUtils.randomiseArray(playlistLength);

			playlistDataArr = HAPUtils.sortArray(playlistDataArr, arr);

		}
		else{
			console.log('Unknown sort order. Sort method failed.');
			return;
		}

		sortOrder = v;

		var current_counter = mediaCounter, orderedItems;

		//reposition data
		playlistContent.append($.map(arr, function(v) {
		    return items[v];
		}));
	
		//update counter
		if(current_counter!=-1){

			var new_counter = playlistContent.children(".hap-playlist-item[data-id='" + current_counter + "']").index();//find new position of active item

			_playlistManager.reSetCounter(new_counter);

		}

		updatePlaylist(true);

	}
	this.destroyInstance = function(){
		if(_window)_window.off(resize_event);
		destroyPlaylist();
		cleanMedia();
		if(usePlaylistScroll){
			playlistInner.mCustomScrollbar('destroy');
			playlistScrollInit = false;
		}
	}
	this.destroyMedia = function() {
		if(!componentInited) return false;
		destroyMedia();
	}
	this.destroyPlaylist = function() {
		if(!componentInited) return false;
		destroyPlaylist();
	}
	this.setPlaybackRate = function(v, preventVisualUpdate){
		if(!componentInited) return false;
		if(!mediaType) return false;

		currMediaData.playbackRate = v;//update playback rate so it doesnt reset on quality change
		if(globalPlaybackRate)globalPlaybackRate = v;
		if(mediaType == 'youtube'){
			youtubePlayer.setPlaybackRate(v);
		}else{
			if(audioUp2Js)audioUp2Js.playbackRate = v;
		}

		if(!preventVisualUpdate)if(_HAPPlaybackRateSlider)_HAPPlaybackRateSlider.setVisual(v);//update slider without trigger change
	}
	this.toggleRandom = function(v) {
		if(!componentInited) return false;
		if(typeof _playlistManager === 'undefined') return false;
		if(typeof v !== 'undefined')randomPlay=v;
		else randomPlay=!randomPlay;
		_playlistManager.setRandom(randomPlay);

		randomToggle.find('.hap-btn').hide();
		if(randomPlay)randomToggle.find('.hap-btn-random-on').show();
		else randomToggle.find('.hap-btn-random-off').show();

	}
	this.setLoop = function(v) {
		if(!componentInited) return false;
		if(typeof _playlistManager === 'undefined') return false;

		loopToggle.find('.hap-btn').hide();
		loopState = v;
		loopStateCounter = loopStateArr.indexOf(loopState);
		loopToggle.find('.hap-btn-loop-'+loopState).show();

		_playlistManager.setLooping(loopState);
	}
	this.getVolume = function() {
		return volume;
	}
	this.setVolume = function(val) {
		if(!componentInited) return false;
		if(val<0) val=0;
		else if(val>1) val=1;
		setVolume(val);
	}
	this.toggleMute = function(){
		if(!componentInited) return false;
		toggleMute();
	}
	this.disableSeek = function(v){
		seekbarDisabled = v;
	}
	this.seek = function(v){
		if(!componentInited) return false;
		if(!mediaType) return false;

		if(mediaType == 'youtube'){
		    if(youtubePlayer && youtubeReady)youtubePlayer.seekTo(v);
		}else if(mediaType == 'audio'){
			if(audioUp2Js){
				try{
					audioUp2Js.currentTime = v;
				}catch(er){console.log(er)}
		    }
		}
	}
	this.seekBackward = function(val){
	    if(!componentInited) return false;
	    if(!mediaType) return false;
	    
	    if(mediaType == 'youtube'){
		    if(youtubePlayer && youtubeReady){
		    	var pos = Math.max(youtubePlayer.getCurrentTime() - (val || seekTime), 0);
		    	youtubePlayer.seekTo(pos);
		    }
		}else if(mediaType == 'audio'){
			if(audioUp2Js){
		    	try{
					audioUp2Js.currentTime = Math.max(audioUp2Js.currentTime - seekTime, 0);
				}catch(er){console.log(er)}
		    }
		}
	}
	this.seekForward = function(val){
	    if(!componentInited) return false;
	    if(!mediaType) return false;
	    
	    if(mediaType == 'youtube'){
	    	if(youtubePlayer && youtubeReady){
				var pos = Math.min(youtubePlayer.getCurrentTime() + (val || seekTime), youtubePlayer.getDuration());
			    youtubePlayer.seekTo(pos);	
			}
		}else if(mediaType == 'audio'){
			if(audioUp2Js){
		    	try{
					audioUp2Js.currentTime = Math.min(audioUp2Js.currentTime + (val || seekTime), audioUp2Js.duration);
				}catch(er){console.log(er)}
		    }
		}
	}
	this.getCurrentTime = function(){
		if(mediaType == 'youtube'){
			if(youtubePlayer && youtubeReady)return youtubePlayer.getCurrentTime();
			else return 0;
		}else if(mediaType == 'audio'){
			if(audioUp2Js)return audioUp2Js.currentTime;
		}else{
			return 0;
		}
	}
	this.getDuration = function(){
		if(!componentInited) return false;
		if(!mediaType) return false;
		
		if(mediaType == 'youtube'){
			if(youtubePlayer && youtubeReady)return youtubePlayer.getDuration();
		}else if(mediaType == 'audio'){
			if(audioUp2Js)return audioUp2Js.duration;
		}
	}
	this.destroyPlaylistScroll = function(){
		if(!componentInited) return false;
		if(usePlaylistScroll){
			playlistInner.mCustomScrollbar('destroy');
			playlistScrollInit = false;
		}
	}
	this.getSetupDone = function(){
		return componentInited;
	}
	this.getMediaPlaying = function(){
		if(!componentInited) return false;
		return mediaPlaying;
	}
	this.getPlaylistLoading = function(){
		return playlistTransitionOn;
	}
	this.getCounter = function(){
		if(componentInited) return typeof _playlistManager !== 'undefined' ? _playlistManager.getCounter() : -1;
		else return null;
	}
	this.getPlaylistContent = function(){
		return playlistContent;	
	}
	this.getPlaylistData = function(){
		return playlistDataArr;	
	}
	this.getPlaylistLength = function(){
		if(!componentInited) return false;
		return HAPUtils.isNumber(playlistLength) ? playlistLength : 0;
	}
	this.getPlaylistList = function(){
		return playlistList;
	}
	this.getSettings = function(){
		return settings;
	}
	this.getCurrMediaData = function(){
		if(!componentInited) return false;
		return currMediaData;	
	}
	this.getWrapper = function(){
		return wrapper;
	}
	this.getPlaylistItems = function(){
		if(!componentInited) return false;
		var arr = [];
		playlistContent.find('.hap-playlist-item').each(function(){
			arr.push($(this));
		});
		return arr;
	}
	this.openPopup = function(){
		if(!componentInited) return false;
		if(settings.isPopup) return false;
		hapOpenPopup(settings, self);
	}
	this.togglePlaylist = function(){

		if(class_list.indexOf('hap-art-narrow-light') > -1 || class_list.indexOf('hap-art-narrow-dark') > -1){

            var time = 350;

            if(!playlistOpened){
                playerHolder.stop().animate({'left': playerHolder.width()+'px'},{duration: time});
                playlistHolder.stop().animate({'left': 0+'px'},{duration: time});
            }else{
                playerHolder.stop().animate({'left': 0+'px'},{duration: time});
                playlistHolder.stop().animate({'left': -playerHolder.width()+'px'},{duration: time});
            }   
            playlistOpened = !playlistOpened;

        }else if(class_list.indexOf('hap-art-wide-light') > -1 || class_list.indexOf('hap-art-wide-dark') > -1 || class_list.indexOf('hap-brona-light') > -1 || class_list.indexOf('hap-brona-dark') > -1){

            var time = 350;
       
            if(!playlistOpened){
                var autoHeight = playlistHolder.css('height', 'auto').height(); 
                playlistHolder.height(0).stop().animate({'height': autoHeight+'px'},{duration: time}, function(){
                    $(this).height('auto');
                });
            }else{
                playlistHolder.stop().animate({'height': 0+'px'},{duration: time});
            }   
            playlistOpened = !playlistOpened;

        }else if(class_list.indexOf('hap-wall') > -1){

        	if(!playlistOpened){
                playlistHolder.css('left',0);
            }else{
                playlistHolder.css('left',-20000+'px');
            }   
            playlistOpened = !playlistOpened;

        }else if(class_list.indexOf('hap-classic') > -1){

            if(playlistOpened){
                playlistHolder.slideUp();
            }else{
                playlistHolder.slideDown();
            }   
            playlistOpened = !playlistOpened;

        }

    }
    this.openPopup = function(){
		if(!componentInited) return false;
		if(settings.isPopup) return false;
		hapOpenPopup(settings, self);
	}
  








	//############################################//
	/* remember playback position */
	//############################################//

	if(hasLocalStorage){

		if(localStorage.getItem(continousKey)){

			var d = JSON.parse(localStorage.getItem(continousKey));
		
			settings.activePlaylist = d.activePlaylist,
			settings.resumeTime = d.resumeTime;
			settings.volume = d.volume;
			settings.activeItem = d.activeItem;
			settings.autoPlay = d.autoPlay;
			autoPlay = d.autoPlay;
			settings.lastPositionArr = d.lastPositionArr;

			localStorage.removeItem(continousKey);
		}
	}

	if(queryPlaylist.length){
		
		//proces playlist from query params

		playlistTransitionOn = true;
		preloader.show();

		$(self).trigger('playlistStartLoad', {instance:self, instanceName:instanceName});

		playlistProcessDataUrl = queryPlaylist;
		playlistLength = playlistProcessDataUrl.length;
		checkProcessCounter();

	}else if(settings.activePlaylist && !HAPUtils.isEmpty(settings.activePlaylist)){
		setPlaylist(settings.activePlaylist);//process playlist
	}else{
		endInit();
	}

	return this;

	}
	
})(jQuery);








//############################################//
/* popup */
//############################################//

var hap_popup_window, hap_player_instance, hasLocalStorage = HAPUtils.hasLocalStorage();  

function hapOpenPopup(settings, inst){

localStorage.removeItem('hap_popup')

	if(hasLocalStorage && localStorage.getItem('hap_popup'))return;

	hap_player_instance = inst;

    var popupUrl = settings.popupUrl || settings.sourcePath + 'popup.html'+"?rand=" + (Math.random() * 99999999), pw = settings.popupWidth || hap_player_instance.width(), ph = settings.popupHeight || hap_player_instance.height(), left = (window.screen.width - pw) / 2, top = (window.screen.height - ph) / 2;
   
    if(!hap_popup_window || hap_popup_window.closed){
        hap_popup_window = window.open(popupUrl,'audio_player','menubar=no,toolbar=no,location=no,scrollbars=1,resizable,width='+pw+',height='+ph+',left='+left+',top='+top+'');

        if(!hap_popup_window) {
            alert("Player can not be opened in a popup window because your browser is blocking Pop-Ups. You need to allow Pop-Ups in browser for this site to use the Player.");
            return false;
        }
       
    }
}

function hapNotifyParent(){//called from popup window when popup window has opened!

    if(hap_player_instance && hap_popup_window && hap_popup_window.initPopup != undefined){

    	/*
    	NOTES:
    	new+copies whole playlist list and loads active playlist agian in player

		old-This copies current playlist from playlist content to popup window (if load more, add more was used etc..)
		actions:
		add more - works
		load more (needs token from before, currently not implemented)
    	*/

        var settings = hap_player_instance.getSettings();

        settings.volume = hap_player_instance.getVolume();
        settings.activePlaylist = settings.activePlaylist;
        settings.activeItem = hap_player_instance.getCounter();
        settings.resumeTime = hap_player_instance.getCurrentTime();
        settings.autoPlay = hap_player_instance.getMediaPlaying();
 
        hap_player_instance.destroyInstance();

        //transfer elements to popup
        var wrapper = hap_player_instance.getWrapper(),
        wrapper_id = wrapper.attr('id'),
        css = hapjq('#hap-inline-css').remove().wrap('<p>').parent().html(),
        playlist_list = hapjq('#hap-playlist-list').remove().wrap('<p>').parent().html();

        //if(HAPUtils.isIE()){//HIERARCHY_REQUEST_ERROR
            wrapper = wrapper.remove().wrap('<p>').parent().html();
        //}

        try {
            window[settings.instance] = hap_popup_window.initPopup(wrapper, wrapper_id, settings, css, playlist_list);
        }catch(e){
            alert('initPopup error: ' + e.message);
            return false;
        }

    }
}


